@@ -20,6 +20,9 @@
 #include "denemo_objects.h"
 #define EXT_MIDI 0
 #define EXT_CSOUND 1
+
+#define DENEMO_DEFAULT_ANON_FTP "ftp://www.denemo.org/download/"
+
 typedef void (*GActionCallback) (GtkAction *action, gpointer data);
 #define G_ACTIONCALLBACK(f) ((GActionCallback)(f)) 
  /* and the following typedefs are basically here for so that it's
@@ -151,6 +154,7 @@ typedef struct
   gint durinticks; /**< Duration of object */
   gint starttick; /**< When the object occurs */ 
   gint starttickofnextnote; /**< When the next object occurs */
+  GList* midi_events;/**< data are the smf_event_ts that this object gives rise to */
   /**< Allots extra space for accidentals or reverse-aligned notes if
    * the stem is down */
   gint space_before; /**< Used to specify how much space is needed before the object */
@@ -193,6 +197,9 @@ typedef GList measurenode;
  */
 typedef struct 
 {
+  GtkMenu *staffmenu; /**< a menu to popup up with the staff directives attached */
+  GtkMenu *voicemenu; /**< a menu to popup up with the voice directives attached */
+
   measurenode *measures; /**< This is a pointer to each measure in the staff */
   clef clef; /**< The initial clef see denemo_objects.h clefs */
   keysig keysig;
@@ -229,7 +236,8 @@ typedef struct
   gint jack_midi_out_port; /**< the port number that jackmidi has assigned */ 
   gint space_above; /**< space above the staff used in the denemo gui */
   gint space_below; /**< space below the staff used in the denemo gui */
-  gboolean haslyrics; /**< TRUE if the staff has had lyrics attached */
+  GList *verses;/**< a list of text editor widgets each containing a verse */
+  GList *currentverse;/**< verse to be displayed */
   gboolean hasfigures; /**<TRUE if the staff has had figures attached. Only one staff should have this set */
   gboolean hasfakechords; /**<TRUE if the staff has had chord symbols attached. Only one staff should have this set */
   gint voicenumber; /**< presently set to 2 for any non-primary voices; we might want to
@@ -303,13 +311,18 @@ typedef struct DenemoPrefs
   gboolean continuous; /*< whether pitch entry overlays should cross barlines */
   gboolean lilyentrystyle;  
   gboolean createclones;
-  gboolean visible_titles;/*< whether the titles should be shown at the top of the score area */
+
   gboolean articulation_palette; /**< This switch makes the articulation pallete visable */
   gboolean notation_palette; /**< This switch makes the duration toolbar visable */
   gboolean rhythm_palette; /**< This option makes the rhythm toolbar visable */
+  gboolean object_palette;  /**< This option makes the object menu toolbar visable */
+
+  gboolean visible_directive_buttons; /**< This option makes the hbox containing score/movement directives visible */
+
   gboolean saveparts; /**< Automatically save parts*/
   gboolean autosave; /**< Auto save data */
   gint autosave_timeout;
+  gboolean autoupdate;/**< update command set from denemo.org */
   gint maxhistory;/**< how long a history of used files to retain */
   GString *browser; /**< Default browser string */
   GString *csoundcommand; /**< command used to execute csound */
@@ -346,11 +359,38 @@ typedef struct DenemoDirective
   gint gx, gy; /**< x and y offsets in pixels for the graphic */
   GString *graphic_name; /**< name of the graphic to be drawn */
   gint width, height; /**< width and height of the bitmap */
+  /* warning these values cannot be changed without bumping the denemo file format version */
 #define DENEMO_OVERRIDE_LILYPOND (1<<0)
+#define DENEMO_OVERRIDE_UNUSED (1<<1)
 #define DENEMO_OVERRIDE_GRAPHIC (1<<2)
-  gint override; /**< specifies what if anything of the built-in behaviour of the object the directive is attached to is to be overriden by this directive */
+
+
+
+#define DENEMO_OVERRIDE_VOLUME (1<<8)
+#define DENEMO_OVERRIDE_DURATION (1<<9)
+#define DENEMO_OVERRIDE_REPEAT (1<<10)
+#define DENEMO_OVERRIDE_CHANNEL (1<<11)
+#define DENEMO_OVERRIDE_TEMPO (1<<12)
+
+#define DENEMO_MIDI_MASK (DENEMO_OVERRIDE_VOLUME | DENEMO_OVERRIDE_DURATION | DENEMO_OVERRIDE_REPEAT | DENEMO_OVERRIDE_CHANNEL | DENEMO_OVERRIDE_TEMPO)
+
+#define DENEMO_OVERRIDE_ONCE (1<<16)
+#define DENEMO_OVERRIDE_STEP (1<<17)
+#define DENEMO_OVERRIDE_RAMP (1<<18)
+
+#define DENEMO_MIDI_ACTION_MASK (DENEMO_OVERRIDE_ONCE | DENEMO_OVERRIDE_STEP | DENEMO_OVERRIDE_RAMP) 
+
+
+#define DENEMO_OVERRIDE_RELATIVE (1<<24)
+#define DENEMO_OVERRIDE_PERCENT (1<<25)
+
+#define DENEMO_MIDI_INTERPRETATION_MASK (DENEMO_OVERRIDE_RELATIVE | DENEMO_OVERRIDE_PERCENT) 
+
+
+
+  guint32 override; /**< specifies what if anything of the built-in behaviour of the object the directive is attached to is to be overriden by this directive and values to use when overriding MIDI */
+  GString *midibytes;/**< values to be used for MIDI generation; the meaning depends fields in override */
   gboolean locked;/**< If true the directive cannot be deleted easily */
-  /* MIDI attributes not done yet */
 } DenemoDirective;
 
 /**
@@ -550,7 +590,7 @@ typedef struct DenemoScore
   
   staffnode *currentprimarystaff;
   staffnode *currentstaff;
-  gint currentstaffnum;
+  gint currentstaffnum;/**< start at 1 */
   measurenode *currentmeasure;
   gint currentmeasurenum;/**< start at 1 */
   objnode *currentobject; /**< currentobject points to the note preceding the cursor when the
@@ -599,12 +639,14 @@ typedef struct DenemoScore
   header header;/*< Directives for the header block of the movement */
   
 
-
+  guint changecount;
   /* Fields used for MIDI playback */
+  gpointer smf;/*< an smf_t structure for libsmf to work with */
   gint tempo;
   gint start;
   gint end;
   gint stafftoplay;
+  guint smfsync;/**< value of changecount when the smf MIDI data was last refreshed */
 
 
   
@@ -625,8 +667,9 @@ typedef struct DenemoScore
   GList *bookmarks;
   gint currentbookmark;
   GList *Instruments;
-  
-}DenemoScore;
+  GtkWidget *buttonbox;/*< box for buttons accessing DenemoDirectives attached to the this movement*/
+  GtkWidget *lyricsbox;/*< box for notebooks containing verses of lyrics for the movement */
+} DenemoScore;
 
 /**
  * DenemoGUI representing a musical score, with associated top level
@@ -648,7 +691,8 @@ typedef struct DenemoGUI
   GtkWidget *printvscrollbar;/**< scrollbar widget for printarea */
   GtkWidget *printhscrollbar;/**< scrollbar widget for printarea */
   GdkPixbuf *pixbuf;/**< print preview pixbuf */
-
+  GtkWidget *buttonboxes;/**< box for boxes showing directives */
+  GtkWidget *buttonbox;/**< box for buttons accessing DenemoDirectives attached to the whole score */
 
   gchar *xbm; /**< xbm representation of graphic bitmap from selected rectangle in print preview area*/
   gint xbm_width, xbm_height;/**< width and height of the xbm data */
@@ -684,8 +728,8 @@ typedef struct DenemoGUI
   GString *filename;/**< the filename to save to */
   GString *autosavename;/**< the filename to autosave to, full path */
 
- 
-  guint changecount;/**< number of edits since last save */
+  gboolean notsaved;/**< edited since last save */
+  guint changecount;/**< number of edits since score loaded */
   guint lilysync;/**< value of changecount when the Lily text was last refreshed */
 
  
