@@ -470,12 +470,14 @@ voiceup (DenemoScriptParam *param)
   if(!gui->si->currentstaff)
     return param->status = FALSE;
   if (gui->si->currentstaff && (((DenemoStaff *)(gui->si->currentstaff->data))->voicenumber==2))  {
-      gui->si->currentstaffnum--;
-      gui->si->currentstaff = gui->si->currentstaff->prev;
-      setcurrentprimarystaff (gui->si);
-      setcurrents (gui->si);
-      move_viewport_down (gui);
-      return param->status = TRUE;
+    hide_lyrics();
+    gui->si->currentstaffnum--;
+    gui->si->currentstaff = gui->si->currentstaff->prev;
+    setcurrentprimarystaff (gui->si);
+    setcurrents (gui->si);
+    show_lyrics();
+    move_viewport_down (gui);
+    return param->status = TRUE;
     } else
       if(param==&dummy)//is interactive
 	warningdialog("This is the first voice");
@@ -501,10 +503,12 @@ staffup (DenemoScriptParam *param)
     voiceup(param);//FIXME check param->status
   if (gui->si->currentstaff->prev)
     {
+      hide_lyrics();
       gui->si->currentstaffnum--;
       gui->si->currentstaff = gui->si->currentstaff->prev;
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
+      show_lyrics();
       find_leftmost_allcontexts (si);
       gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME gets cursorclef set
       move_viewport_up (gui);
@@ -532,10 +536,12 @@ voicedown (DenemoScriptParam *param)
   if(!gui->si->currentstaff)
     return param->status = FALSE;
   if (gui->si->currentstaff->next && ((DenemoStaff *)(gui->si->currentstaff->next->data))->voicenumber==2) {
+      hide_lyrics();
       gui->si->currentstaffnum++;
       gui->si->currentstaff = gui->si->currentstaff->next;
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
+      show_lyrics();
       move_viewport_down (gui);
       return param->status = TRUE;
     } else
@@ -563,10 +569,12 @@ staffdown (DenemoScriptParam *param)
     voicedown(param);//FIXME
   if (gui->si->currentstaff->next)
     {
+      hide_lyrics();
       gui->si->currentstaffnum++;
       gui->si->currentstaff = gui->si->currentstaff->next;
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
+      show_lyrics();
       find_leftmost_allcontexts (si);
       gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME gets cursorclef set
       move_viewport_down (gui);
@@ -880,7 +888,7 @@ dnm_insertchord (DenemoGUI * gui, gint duration, input_mode mode,
     return;
   }
   insertion_point (si);
-  
+
   /* Now actually create the chord */
   mudela_obj_new = newchord (duration, 0, 0);
   if ((mode & INPUTNORMAL) && (rest != TRUE))
@@ -902,9 +910,9 @@ dnm_insertchord (DenemoGUI * gui, gint duration, input_mode mode,
  
   if (gui->mode&(INPUTRHYTHM)) {
     if(rest)
-      play_pitch(64.0*(1+duration), 60.0/(4.0*gui->si->tempo*(1<<duration)), 0.1, 2);//FIXME make a distinct noise
+      playpitch(64.0*(1+duration), 60.0/(4.0*gui->si->tempo*(1<<duration)), 0.1, 2);//FIXME make a distinct noise
       else
-	play_pitch(64.0*(1+duration), 60.0/(gui->si->tempo*(1<<duration)), 0.2, 1);
+	playpitch(64.0*(1+duration), 60.0/(gui->si->tempo*(1<<duration)), 0.2, 1);
   } else {
     if(Denemo.gui->input_source==INPUTKEYBOARD) {
       DenemoStaff *curstaffstruct = (DenemoStaff *) si->currentstaff->data;
@@ -1200,7 +1208,7 @@ changedots (DenemoScore * si, gint amount)
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
       if (Denemo.gui->mode&(INPUTRHYTHM))
-	  play_pitch(440.0, 0.2, 0.2, 1);
+	  playpitch(440.0, 0.2, 0.2, 1);
       changenumdots (curmudelaobj, amount);
 
       if (curmudelaobj->user_string)
