@@ -15,7 +15,7 @@
 #include "staffops.h"
 
 /**
- * This function finds the first DenemoObject of type thetype before measure
+ * This function finds the first DenemoObject of type thetype *before* measure
  * curmeasure. It returns NULL if it was unable to find such a DenemoObject 
  * 
  * @param curmeasure the current measure to search
@@ -23,7 +23,7 @@
  * @return the first DenemoObject of type the type
  */
 static DenemoObject *
-find_context (measurenode * curmeasure, gint thetype)
+find_measure_context (measurenode * curmeasure, gint thetype)
 {
   objnode *curobj;
   if (curmeasure && curmeasure->prev)
@@ -88,9 +88,9 @@ gint
 find_prevailing_clef(DenemoScore *si) {
 DenemoStaff *curstaff = ((DenemoStaff*)si->currentstaff->data);
 DenemoObject *obj= find_context_of_object (si, CLEF);
-// g_print("prevailing clef %d\n",  obj? ((clef *) obj->object)->type:curstaff->sclef);
+// g_print("prevailing clef %d\n",  obj? ((clef *) obj->object)->type:curstaff->clef.type);
 return obj? ((clef *) obj->object)->type:
-  curstaff->sclef;
+  curstaff->clef.type;
 }
 
 /**
@@ -108,34 +108,31 @@ find_leftmost_staffcontext (DenemoStaff * curstaffstruct, DenemoScore * si)
 					 si->leftmeasurenum - 1);
   DenemoObject *obj;
 
-  if ((obj = find_context (leftmeasure, CLEF)))
-    curstaffstruct->leftmost_clefcontext = ((clef *) obj->object)->type;
+  if ((obj = find_measure_context (leftmeasure, CLEF)))
+    curstaffstruct->leftmost_clefcontext = ((clef *) obj->object);
   else
-    curstaffstruct->leftmost_clefcontext = curstaffstruct->sclef;
-  if ((obj = find_context (leftmeasure, KEYSIG)))
-    curstaffstruct->leftmost_keysigcontext = ((keysig *) obj->object)->number;
+    curstaffstruct->leftmost_clefcontext = &curstaffstruct->clef;
+  if ((obj = find_measure_context (leftmeasure, KEYSIG)))
+    curstaffstruct->leftmost_keysig = (keysig *) obj->object;
   else
-    curstaffstruct->leftmost_keysigcontext = curstaffstruct->skey;
-  /*  dnm_setinitialkeysig  (curstaffstruct, curstaffstruct->leftmost_keysigcontext,
-      curstaffstruct->skey_isminor);*/
-  initkeyaccs (curstaffstruct->leftmost_keyaccs,
-	       curstaffstruct->leftmost_keysigcontext);
-  curstaffstruct->leftmost_keywidth =
-    draw_key (NULL, NULL, 0, 0, curstaffstruct->leftmost_keysigcontext,
-	      0, 0, FALSE);
-  if ((obj = find_context (leftmeasure, TIMESIG)))
+    curstaffstruct->leftmost_keysig = &curstaffstruct->keysig;
+
+  // initkeyaccs (curstaffstruct->leftmost_keysig->accs,
+  //	       curstaffstruct->leftmost_keysig->number);
+
+      si->maxkeywidth = MAX (si->maxkeywidth, 
+			     draw_key (NULL, NULL, 0, 0, curstaffstruct->leftmost_keysig->number,
+	      0, 0, FALSE));
+  if ((obj = find_measure_context (leftmeasure, TIMESIG)))
     {
-      curstaffstruct->leftmost_time1context =
-	((timesig *) obj->object)->time1;
-      curstaffstruct->leftmost_time2context =
-	((timesig *) obj->object)->time2;
+      curstaffstruct->leftmost_timesig =
+	(timesig *) obj->object;
     }
   else
     {
-      curstaffstruct->leftmost_time1context = curstaffstruct->stime1;
-      curstaffstruct->leftmost_time2context = curstaffstruct->stime2;
+      curstaffstruct->leftmost_timesig = &curstaffstruct->timesig;
     }
-  if ((obj = find_context (leftmeasure, STEMDIRECTIVE)))
+  if ((obj = find_measure_context (leftmeasure, STEMDIRECTIVE)))
     curstaffstruct->leftmost_stem_directive =
       ((stemdirective *) obj->object)->type;
   else
@@ -157,8 +154,6 @@ find_leftmost_allcontexts (DenemoScore * si)
   for (; curstaff; curstaff = curstaff->next)
     {
       find_leftmost_staffcontext ((DenemoStaff *) curstaff->data, si);
-      si->maxkeywidth = MAX (si->maxkeywidth,
-			     ((DenemoStaff *) curstaff->data)->
-			     leftmost_keywidth);
+
     }
 }
