@@ -27,8 +27,25 @@
  * depending upon the GTK version
  *
  */
-#define REDEXCL_WIDTH 3
-#define REDEXCL_HEIGHT 13
+#define EXCL_WIDTH 3
+#define EXCL_HEIGHT 13
+
+static GdkGC *  bluegc;
+static GdkGC *  redgc;
+static GdkGC *  graygc;
+static GdkGC *  slategraygc;
+static GdkGC *  lightbluegc;
+static GdkGC *  blackgc;
+static GdkGC *greengc = NULL;
+
+GdkPixbuf *StaffPixbuf;
+
+static void      
+create_tool_pixbuf(void) {
+  GtkWidget *widget = gtk_button_new();
+  StaffPixbuf = gtk_widget_render_icon (widget, GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_BUTTON, "denemo");
+}
+
 
 /**
  * scorearea_configure_event
@@ -41,6 +58,15 @@ gint
 scorearea_configure_event (GtkWidget * widget, GdkEventConfigure * event)
 {
   DenemoGUI *gui = Denemo.gui;
+  bluegc = gcs_bluegc();
+  redgc = gcs_redgc();
+  graygc = gcs_graygc();
+  slategraygc = gcs_slategraygc();
+  lightbluegc = gcs_lightbluegc();
+  blackgc = gcs_blackgc();
+  greengc = gcs_greengc();
+  create_tool_pixbuf();
+
   /* Create a new backing pixmap of the appropriate size */
   if (gui->pixmap)
     gdk_pixmap_unref (gui->pixmap);
@@ -84,7 +110,7 @@ struct infotopass
   GSList *slur_stack;
   GSList *hairpin_stack;
   GdkGC *gc;
-  gboolean haslyrics;
+  GtkWidget * verse;
   gint space_above;
   gint highy;/*(return) the highest y value drawn*/
   gint lowy;/*(return) the lowest y value drawn*/
@@ -93,6 +119,31 @@ struct infotopass
   gboolean mark;//whether the region is selected
 };
 
+/* count the number of syllables up to staff->leftmeasurenum */
+static gint count_syllables(DenemoStaff *staff, gint from) {
+  gint count = 0;
+  gint i;
+  GList *curmeasure = staff->measures;
+  for(i=1;i<from;i++, curmeasure = curmeasure->next) {
+    objnode *curobj;
+    for(curobj = curmeasure->data;curobj;curobj=curobj->next) {
+      DenemoObject *obj = curobj->data;
+      gboolean in_slur = FALSE;
+      if(obj->type==CHORD) {
+	  chord *thechord = ((chord *) obj->object);
+	  if(thechord->notes && !in_slur)
+	    count++;
+	  if (thechord->slur_begin_p)
+	    in_slur = TRUE;
+	  if (thechord->slur_end_p)
+	    in_slur = FALSE;
+      }
+    }
+  }
+  return count;
+}
+
+
 /**
  *  draw_object
  *
@@ -108,9 +159,8 @@ static gboolean
 draw_object (objnode * curobj, gint x, gint y,
 	     DenemoGUI * gui, struct infotopass *itp)
 {
-  static GdkGC *blackgc = NULL;
-  static GdkGC *redgc = NULL;
-  static GdkGC *greengc = NULL;
+
+
   static gboolean init=FALSE;
   static GdkColor white, black, blue, green, yellow;
   if(!init) {
@@ -132,11 +182,7 @@ draw_object (objnode * curobj, gint x, gint y,
   /* The current note, rest, etc. being painted */
   gint extra;
 
-  if (!blackgc)
-    blackgc = gcs_blackgc ();
 
-  if (!redgc)
-    redgc = gcs_redgc ();
   if (!greengc)
     greengc = gcs_greengc ();
   /* Should we set cursor-context info before drawing? */
@@ -163,8 +209,9 @@ draw_object (objnode * curobj, gint x, gint y,
   switch (mudelaitem->type)
     {
     case CHORD:
-       if (((chord *) mudelaitem->object)->is_figure && ((chord *) mudelaitem->object)->figure)
-      //if (((chord *) mudelaitem->object)->is_figure)
+      { chord *thechord = ((chord *) mudelaitem->object);
+       if (thechord->is_figure && thechord->figure)
+      //if (thechord->is_figure)
 	//draw_figure (gui->pixmap, itp->gc,
 	//	     gtk_style_get_font (itp->widget->style),
 	//	     x + mudelaitem->x, y, mudelaitem);
@@ -172,7 +219,7 @@ draw_object (objnode * curobj, gint x, gint y,
 	draw_figure (gui->pixmap, itp->gc,
 		     gtk_style_get_font (itp->widget->style),
 		     x + mudelaitem->x,
-		     y + (((chord *) mudelaitem->object)->lowesty / 2),
+		     y + (thechord->lowesty / 2),
 		     mudelaitem);
   
       else
@@ -180,22 +227,22 @@ draw_object (objnode * curobj, gint x, gint y,
 	  draw_chord (gui->pixmap, itp->gc, curobj, x + mudelaitem->x, y,
 		      GPOINTER_TO_INT (itp->mwidthiterator->data),
 		      itp->curaccs, itp->mark);
-	  if((((chord *) mudelaitem->object)->highesty) < itp->highy)
- 	    itp->highy  = ((chord *) mudelaitem->object)->highesty/*, g_print("setting highy %d\n", itp->highy)*/;
+	  if((thechord->highesty) < itp->highy)
+ 	    itp->highy  = thechord->highesty/*, g_print("setting highy %d\n", itp->highy)*/;
 
-	  if((((chord *) mudelaitem->object)->lowesty) > itp->lowy+STAFF_HEIGHT)
-	    itp->lowy  = ((chord *) mudelaitem->object)->lowesty-STAFF_HEIGHT;
+	  if((thechord->lowesty) > itp->lowy+STAFF_HEIGHT)
+	    itp->lowy  = thechord->lowesty-STAFF_HEIGHT;
 
 
 
 
 	}
-       if (((chord *) mudelaitem->object)->is_fakechord)
+       if (thechord->is_fakechord)
 	draw_fakechord (gui->pixmap, itp->gc,
 		     gtk_style_get_font (itp->widget->style),
 		     x + mudelaitem->x, 
 		     y - 45,
-		     //y - (((chord *) mudelaitem->object)->highesty ), 
+		     //y - (thechord->highesty ), 
 		     mudelaitem);
        else
 	{
@@ -204,41 +251,42 @@ draw_object (objnode * curobj, gint x, gint y,
 		      itp->curaccs, itp->mark);
 	}
 
-      if (((chord *) mudelaitem->object)->lyric)
+       if (si->currentstaffnum==itp->staffnum && itp->verse && thechord->notes   && !itp->slur_stack )
 	{
-	  draw_lyric (gui->pixmap, itp->gc,
-		      gtk_style_get_font (itp->widget->style),
-		      x + mudelaitem->x,
-		      y + ((chord *) mudelaitem->object)->lowesty,
-		      mudelaitem);
-	  itp->haslyrics = TRUE;
+	  gchar *syllable = next_syllable();
+	  if(syllable)
+	    draw_lyric (gui->pixmap, itp->gc,
+			gtk_style_get_font (itp->widget->style),
+			x + mudelaitem->x,
+			y + thechord->lowesty,
+			syllable);
 	}
 
-      if (((chord *) mudelaitem->object)->dynamics)
+      if (thechord->dynamics)
 	draw_dynamic (gui->pixmap, itp->gc,
 		      gtk_style_get_font (itp->widget->style),
 		      x + mudelaitem->x, y, mudelaitem);
 
-      if (((chord *) mudelaitem->object)->slur_end_p)
+      if (thechord->slur_end_p)
 	draw_slur (gui->pixmap, itp->gc, &(itp->slur_stack),
 		   x + mudelaitem->x, y);
-      if (((chord *) mudelaitem->object)->slur_begin_p)
+      if (thechord->slur_begin_p)
 	itp->slur_stack =
 	  push_slur_stack (itp->slur_stack, x + mudelaitem->x);
 
-      if (((chord *) mudelaitem->object)->crescendo_begin_p)
+      if (thechord->crescendo_begin_p)
 	itp->hairpin_stack =
 	  push_hairpin_stack (itp->hairpin_stack, x + mudelaitem->x);
-      else if (((chord *) mudelaitem->object)->diminuendo_begin_p)
+      else if (thechord->diminuendo_begin_p)
 	itp->hairpin_stack =
 	  push_hairpin_stack (itp->hairpin_stack, x + mudelaitem->x);
-      if (((chord *) mudelaitem->object)->crescendo_end_p)
+      if (thechord->crescendo_end_p)
 	{
 	  if (top_hairpin_stack (itp->hairpin_stack) <= -1)
 	    {
 #if 0
 	      //this is only the visible part of the cresc, the start may be off screen
-	      ((chord *) mudelaitem->object)->crescendo_end_p = FALSE;
+	      thechord->crescendo_end_p = FALSE;
 	      warningdialog
 		("Crescendo end without a corresponding start\n"
 		 "removing the crescendo end");
@@ -247,13 +295,13 @@ draw_object (objnode * curobj, gint x, gint y,
 	  draw_hairpin (gui->pixmap, itp->gc, &(itp->hairpin_stack),
 			x + mudelaitem->x, y, 1);
 	}
-      else if (((chord *) mudelaitem->object)->diminuendo_end_p)
+      else if (thechord->diminuendo_end_p)
 	{
 	  if (top_hairpin_stack (itp->hairpin_stack) <= -1)
 	    {
 #if 0
 	      //this is only the visible part of the dim, the start may be off screen
-	      ((chord *) mudelaitem->object)->diminuendo_end_p = FALSE;
+	      thechord->diminuendo_end_p = FALSE;
 	      warningdialog
 		("Diminuendo end without a corresponding start\n"
 		 "removing the diminuendo end");
@@ -263,21 +311,21 @@ draw_object (objnode * curobj, gint x, gint y,
 			x + mudelaitem->x, y, 0);
 	}
 	/* notice the following does not check is_figure but checks if figure is not VOID) */      
-      if (!((chord *) mudelaitem->object)->is_figure && ((chord *) mudelaitem->object)->figure)
-      //if (((chord *) mudelaitem->object)->figure)
+      if (!thechord->is_figure && thechord->figure)
+      //if (thechord->figure)
         draw_figure (gui->pixmap, itp->gc,
 		     gtk_style_get_font (itp->widget->style),
 		     x + mudelaitem->x,
-		     y + (((chord *) mudelaitem->object)->lowesty / 2),
+		     y + (thechord->lowesty / 2),
 		     mudelaitem);
-      if (!((chord *) mudelaitem->object)->is_fakechord && ((chord *) mudelaitem->object)->fakechord) 
+      if (!thechord->is_fakechord && thechord->fakechord) 
 	draw_fakechord (gui->pixmap, itp->gc,
 		     gtk_style_get_font (itp->widget->style),
 		     x + mudelaitem->x,
 		     y - 45,
-		     //y + (((chord *) mudelaitem->object)->highesty / 4),
+		     //y + (thechord->highesty / 4),
 		     mudelaitem);
-
+      }
       break;
     case TUPOPEN:
     case TUPCLOSE:
@@ -397,7 +445,7 @@ draw_object (objnode * curobj, gint x, gint y,
   gdk_gc_set_foreground (blackgc, &black);
   //g_print("returning with %d\n", itp->highy);
   /* And give a return value and we're done */
-  return (mudelaitem->starttickofnextnote > itp->tickspermeasure);
+  return (mudelaitem->starttickofnextnote - itp->tickspermeasure);
 } /* draw_object */
 
 /**
@@ -412,8 +460,7 @@ void
 draw_measure (measurenode * curmeasure, gint x, gint y,
 	      DenemoGUI * gui, struct infotopass *itp)
 {
-  static GdkPixmap *redexclaim = NULL;
-  static GdkGC *redgc;
+  static GdkPixmap *exclam = NULL;
   static GString *mstring;
   static PangoContext *context;
   static PangoLayout *layout;
@@ -429,11 +476,10 @@ draw_measure (measurenode * curmeasure, gint x, gint y,
       layout = pango_layout_new (context);
     }
   /* initialization */
-  if (!redexclaim)
+  if (!exclam)
     {
-      redexclaim = bitmaphelper (itp->widget, toomany);
+      exclam = bitmaphelper (itp->widget, toomany);
       mstring = g_string_new (NULL);
-      redgc = gcs_redgc ();
     }
 
   /* Set information about the state at the current measure,
@@ -526,11 +572,12 @@ draw_measure (measurenode * curmeasure, gint x, gint y,
        );
     offend = draw_object (curobj, x, y, gui, itp);
   }
-  /* Paint the red exclamation point, if necessary */
-  if (offend)
-    drawbitmapinverse (gui->pixmap, redgc, redexclaim,
-		       x, y - 8 - REDEXCL_HEIGHT,
-		       REDEXCL_WIDTH, REDEXCL_HEIGHT);
+  /* Paint the exclamation point, if necessary */
+  GdkGC *exclamgc = (offend>0)?redgc:(offend<0)?bluegc:NULL;
+  if(exclamgc)
+    drawbitmapinverse (gui->pixmap, exclamgc, exclam,
+		       x, y - 8 - EXCL_HEIGHT,
+		       EXCL_WIDTH, EXCL_HEIGHT);
 }
 
 /**
@@ -538,7 +585,7 @@ draw_measure (measurenode * curmeasure, gint x, gint y,
  * TODO sort out graphics context for active polyphonic voice should 
  * do it here
  * @param curstaffstruct pointer to the staff to draw
- * @param y    y poisition of the staff
+ * @param y    y position of the staff
  * @param gui   pointer to the DenemoGUI structure
  * @param itp  pointer to the infotopass structure
  */
@@ -549,8 +596,6 @@ draw_staff (DenemoStaff * curstaffstruct, gint y,
   PangoContext *context;
   PangoLayout *layout;
   PangoFontDescription *desc;
-  static GdkGC *blackgc;
-  static GdkGC *graygc;
 
   DenemoScore *si = gui->si;
   gint x, i;
@@ -559,15 +604,10 @@ draw_staff (DenemoStaff * curstaffstruct, gint y,
     gdk_pango_context_get_for_screen (gdk_drawable_get_screen (gui->pixmap));
   layout = pango_layout_new (context);
 
-  if (!blackgc)
-    {
-      blackgc = gcs_blackgc ();
-      graygc = gcs_slategraygc ();
-    }
   if ((DenemoStaff *) si->currentstaff->data == curstaffstruct)
     gc = blackgc;
   else
-    gc = graygc;
+    gc = slategraygc;
 
 
 
@@ -604,25 +644,6 @@ draw_staff (DenemoStaff * curstaffstruct, gint y,
 
   gdk_draw_layout (gui->pixmap, gc, KEY_MARGIN, y - buffer, layout);
   gint title_highy = 0;
-  if(curstaffstruct->staff_directives) {
-    GList *g = curstaffstruct->staff_directives;
-    gint count=1;
-    for(;g;g=g->next, count++) {
-      DenemoDirective* directive = g->data;
-        if(directive->display) { 
-    pango_layout_set_text (layout,
-			   directive->display->str,
-			   -1);
-    pango_layout_set_font_description (layout, desc);
-    gint ypos = y-buffer+directive->ty - count*10;
-    gdk_draw_layout (gui->pixmap, gc, KEY_MARGIN + directive->tx, ypos, layout);
-    //g_print("high %d %d\n", itp->highy, -ypos);
-    if(title_highy > ypos) title_highy = ypos;
-  }
-    }
-
-  }
-  //FIXME voice directives...
 
   pango_font_description_free (desc);
 
@@ -643,7 +664,7 @@ draw_staff (DenemoStaff * curstaffstruct, gint y,
 	 && itp->measurenum <= g_list_length (curstaffstruct->measures))
     {
       if(itp->measurenum == si->rightmeasurenum+1)
-	itp->gc = gcs_slategraygc ();
+	itp->gc = slategraygc;
       draw_measure (itp->curmeasure, x, y, gui, itp);
       x += GPOINTER_TO_INT (itp->mwidthiterator->data) + SPACE_FOR_BARLINE;
       itp->curmeasure = itp->curmeasure->next;
@@ -668,47 +689,6 @@ draw_staff (DenemoStaff * curstaffstruct, gint y,
     }
 }
 
-/**
- * Draw the score directives and return the amount of vertical space they took up.
-
-*/
-static gint
-draw_score_directives (void) {
-  GList *score_directives = Denemo.gui->lilycontrol.directives;
-  GList *scoreheader_directives = Denemo.gui->scoreheader.directives;
-  GList *header_directives = Denemo.gui->si->header.directives;
-  GList *paper_directives = Denemo.gui->paper.directives;
-  GList *layout_directives = Denemo.gui->si->layout.directives;
-  GList *movement_directives = Denemo.gui->si->movementcontrol.directives;
-  gint x=0, y = 0;
-
-  PangoContext *context =
-    gdk_pango_context_get_for_screen (gdk_drawable_get_screen (Denemo.gui->pixmap));
-  PangoLayout *layout = pango_layout_new (context);
-  PangoFontDescription *desc = pango_font_description_from_string (FONT);
-#define OUTPUT_DIREC(g)\
-  for(g?y+=10:1;g;g=g->next) {\
-   DenemoDirective *directive = g->data;\
-    if(directive->display) {\
-      pango_layout_set_text (layout,\
-			     directive->display->str,\
-			     -1);\
-      pango_layout_set_font_description (layout, desc);\
-      gdk_draw_layout (Denemo.gui->pixmap, gcs_blackgc(), x, y, layout);\
-      x += strlen(directive->display->str) * MAX(10, directive->tx);\
-    }\
-  }
-  OUTPUT_DIREC(score_directives);
-  OUTPUT_DIREC(scoreheader_directives);
-  OUTPUT_DIREC(header_directives);
-  OUTPUT_DIREC(paper_directives);
-  OUTPUT_DIREC(layout_directives);
-  OUTPUT_DIREC(movement_directives);
-#undef OUTPUT_DIREC
-  pango_font_description_free (desc);
-  return y;
-}
-
 /**
  * This actually draws the score, staff-by-staff 
  * @param widget pointer to the parent widget
@@ -721,122 +701,119 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
   staffnode *curstaff;
   gint x, y=0, i;
   struct infotopass itp;
-  static GdkGC *blackgc = NULL;
-  static GdkGC *bluegc;
-  static GdkGC *graygc;
+
   GList *mwidthiterator;
   gboolean repeat = FALSE;
   DenemoScore *si = gui->si;
 
-  /* One-time inits */
-  if (!blackgc)
-    {
-      blackgc = gcs_blackgc ();
-      bluegc = gcs_bluegc ();
-      graygc = gcs_graygc ();
-    }
 
   /* Initialize some fields in itp */
 
   itp.widget = widget;// this is used for widget->style, it is spuriously passed to the macro bitmaphelper
   itp.slur_stack = NULL;
   itp.hairpin_stack = NULL;
-  itp.haslyrics = FALSE;
+
   itp.highy = 0;//in case there are no objects...
   y = 0;
-  //draw score title etc above top staff, if it is visible and if desired by score directives
-  if(si->top_staff==1 && Denemo.prefs.visible_titles) {
-    static last_y;
-    curstaff = si->thescore;
-    gint space = ((DenemoStaff *) curstaff->data)->space_above;
-    do {
-      last_y = y;
-      y = 0;
-      gdk_draw_rectangle (gui->pixmap, gcs_lightbluegc(), TRUE, 0, 0, widget->allocation.width/*KEY_MARGIN*/, last_y);
-      y = draw_score_directives();  
-      if(space<y) 
-	space = ((DenemoStaff *) curstaff->data)->space_above = y;
-    } while (last_y != y);
-  }
+
+
   /* Draw each staff */
   for ((itp.staffnum = si->top_staff,
 	curstaff = g_list_nth (si->thescore, si->top_staff - 1),
 	y += si->staffspace / 4);
-       curstaff && itp.staffnum <= si->bottom_staff; itp.staffnum++)
-    {
-      if (curstaff && ((DenemoStaff *) curstaff->data)->voicenumber == 1)
-	y += ((DenemoStaff *) curstaff->data)->space_above;
-      itp.space_above = ((DenemoStaff *) curstaff->data)->space_above;
-      gint top_y = (si->staffspace / 4) + itp.space_above;
+       curstaff && itp.staffnum <= si->bottom_staff; itp.staffnum++) {
+    DenemoStaff *staff = (DenemoStaff *) curstaff->data;
+    itp.verse = staff->currentverse?staff->currentverse->data:NULL;
+
+    if (curstaff && staff->voicenumber == 1)
+      y += staff->space_above;
+    itp.space_above = staff->space_above;
+    gint top_y = (si->staffspace / 4) + itp.space_above;
     
-      itp.top_y = top_y;
-      itp.y = y;
-      gint highy = ((DenemoStaff *) curstaff->data)->space_above;
-      gint lowy =  ((DenemoStaff *) curstaff->data)->space_below;
+    itp.top_y = top_y;
+    itp.y = y;
+    gint highy = staff->space_above;
+    gint lowy =  staff->space_below;
 
-      itp.in_highy = highy, itp.in_lowy = lowy;
-      itp.highy = 0;//do not pass on extra_space from one staff to the next
+    itp.in_highy = highy, itp.in_lowy = lowy;
+    itp.highy = 0;//do not pass on extra_space from one staff to the next
+	gdk_draw_rectangle (gui->pixmap, gcs_lightbluegc(), TRUE, 0, y, LEFT_MARGIN, STAFF_HEIGHT/*staff edit*/);
+    if(curstaff==si->currentstaff) {
+      if(staff->staff_directives) {
 
-      gdk_draw_rectangle (gui->pixmap, gcs_lightbluegc(), TRUE, 0, y, LEFT_MARGIN, STAFF_HEIGHT/*staff edit*/);
-      if(si->leftmeasurenum==1) {
-	/* draw background of clef, keysig, timesig */
-	gint key = gui->si->maxkeywidth;
-	gint cmajor = key?0:5;//allow some area for keysig in C-major
-	gdk_draw_rectangle (gui->pixmap, gcs_graygc(), TRUE, LEFT_MARGIN,y,KEY_MARGIN-LEFT_MARGIN - cmajor,STAFF_HEIGHT);/*clef edit*/
-	gdk_draw_rectangle (gui->pixmap, gcs_lightbluegc(), TRUE, KEY_MARGIN-cmajor,y,key+2*cmajor,STAFF_HEIGHT);/*keysig edit*/
-	gdk_draw_rectangle (gui->pixmap, gcs_graygc(), TRUE, KEY_MARGIN+key+cmajor,y,SPACE_FOR_TIME-cmajor,STAFF_HEIGHT);/*timesig edit*/
+	guint width = gdk_pixbuf_get_width( GDK_PIXBUF(StaffPixbuf));
+	guint height = gdk_pixbuf_get_height( GDK_PIXBUF(StaffPixbuf));
+	gdk_draw_pixbuf(gui->pixmap, NULL, StaffPixbuf,  0,0, 0,y, width, height, GDK_RGB_DITHER_NONE,0,0/*staff edit*/);
       }
-      draw_staff ((DenemoStaff *) curstaff->data, y, gui, &itp);
+      if(staff->voice_directives) {
 
-      //IN FACT itp.highy is only set by one measure, it is reset to zero in the measure loop
-      if(-itp.highy>highy && -itp.highy<MAXEXTRASPACE) //FIXME this should be done before draw_staff returns
-	/*g_print("setting space above %d staff %d\n", -itp.highy, itp.staffnum),*/((DenemoStaff *) curstaff->data)->space_above = -itp.highy, repeat=TRUE;
-      if(itp.lowy>lowy && itp.lowy<MAXEXTRASPACE)
-	((DenemoStaff *) curstaff->data)->space_below = itp.lowy, repeat=TRUE;
+	guint width = gdk_pixbuf_get_width( GDK_PIXBUF(StaffPixbuf));
+	guint height = gdk_pixbuf_get_height( GDK_PIXBUF(StaffPixbuf));
+	gdk_draw_pixbuf(gui->pixmap, NULL, StaffPixbuf,  0,0, 0,y + STAFF_HEIGHT/2, width, height, GDK_RGB_DITHER_NONE,0,0/*staff edit*/);
+      }
+    }
+    
 
 
-        /* Now draw the barlines between the measures, across all the staffs */
-       mwidthiterator = g_list_nth (si->measurewidths, si->leftmeasurenum - 1);
-       for (x = KEY_MARGIN + si->maxkeywidth + 
-	      SPACE_FOR_TIME - HALF_BARLINE_SPACE,
-	      i = si->leftmeasurenum;
-	    i <= si->rightmeasurenum; mwidthiterator = mwidthiterator->next, 
-	      i++)
-	 {
-	   gint top = y + STAFF_HEIGHT; 
+    if(si->leftmeasurenum==1) {
+      /* draw background of clef, keysig, timesig */
+      gint key = gui->si->maxkeywidth;
+      gint cmajor = key?0:5;//allow some area for keysig in C-major
+      gdk_draw_rectangle (gui->pixmap, graygc, TRUE, LEFT_MARGIN,y,KEY_MARGIN-LEFT_MARGIN - cmajor,STAFF_HEIGHT);/*clef edit*/
+      gdk_draw_rectangle (gui->pixmap, lightbluegc, TRUE, KEY_MARGIN-cmajor,y,key+2*cmajor,STAFF_HEIGHT);/*keysig edit*/
+      gdk_draw_rectangle (gui->pixmap, graygc, TRUE, KEY_MARGIN+key+cmajor,y,SPACE_FOR_TIME-cmajor,STAFF_HEIGHT);/*timesig edit*/
+    }
+    if(si->currentstaffnum==itp.staffnum)
+      reset_lyrics(staff, count_syllables(staff, si->leftmeasurenum));
+    draw_staff (staff, y, gui, &itp);
 
-	   x += GPOINTER_TO_INT (mwidthiterator->data) + SPACE_FOR_BARLINE;
+    //IN FACT itp.highy is only set by one measure, it is reset to zero in the measure loop
+    if(-itp.highy>highy && -itp.highy<MAXEXTRASPACE) //FIXME this should be done before draw_staff returns
+      /*g_print("setting space above %d staff %d\n", -itp.highy, itp.staffnum),*/staff->space_above = -itp.highy, repeat=TRUE;
+    if(itp.lowy>lowy && itp.lowy<MAXEXTRASPACE)
+      staff->space_below = itp.lowy, repeat=TRUE;
+
+
+    /* Now draw the barlines between the measures, across all the staffs */
+    mwidthiterator = g_list_nth (si->measurewidths, si->leftmeasurenum - 1);
+    for (x = KEY_MARGIN + si->maxkeywidth + 
+	   SPACE_FOR_TIME - HALF_BARLINE_SPACE,
+	   i = si->leftmeasurenum;
+	 i <= si->rightmeasurenum; mwidthiterator = mwidthiterator->next, 
+	   i++)
+      {
+	gint top = y + STAFF_HEIGHT; 
+
+	x += GPOINTER_TO_INT (mwidthiterator->data) + SPACE_FOR_BARLINE;
 	   
-	   if (!mwidthiterator->next) /* Last measure - draw double-barline */
-	     x -= 3;
-	   gdk_draw_line (gui->pixmap, graygc, x, top, x, y);
+	if (!mwidthiterator->next) /* Last measure - draw double-barline */
+	  x -= 3;
+	gdk_draw_line (gui->pixmap, graygc, x, top, x, y);
 	   
-	   if (!mwidthiterator->next)
-	     {
-	       /* Again, we've reached the end of the score and should
-		* draw a double-barline */
-	       x += 3;
-	       gdk_draw_rectangle (gui->pixmap, blackgc, TRUE, x,
-				   y, 4,
-				   STAFF_HEIGHT+1);
-	     }
+	if (!mwidthiterator->next)
+	  {
+	    /* Again, we've reached the end of the score and should
+	     * draw a double-barline */
+	    x += 3;
+	    gdk_draw_rectangle (gui->pixmap, blackgc, TRUE, x,
+				y, 4,
+				STAFF_HEIGHT+1);
+	  }
 	   
-	 }
+      }
 
-       if ( itp.staffnum < si->bottom_staff
-	   &&    ((DenemoStaff *) curstaff->next->data)->voicenumber !=2)
-	 {
-	   if (itp.haslyrics) {
-	     if(!((DenemoStaff *) curstaff->data)->haslyrics)
-	       ((DenemoStaff *) curstaff->data)->haslyrics = TRUE;//, repeat=TRUE;
-	     y += LYRICS_HEIGHT;
-	   }
-	   y +=
-	     (si->staffspace + ((DenemoStaff *) curstaff->data)->space_below);
-	 }
-       itp.haslyrics = FALSE;
-       curstaff = curstaff->next;
-    }// for all the staffs
+    if ( itp.staffnum < si->bottom_staff
+	 &&    ((DenemoStaff *) curstaff->next->data)->voicenumber !=2)
+      {
+	if (itp.verse) {
+	  y += LYRICS_HEIGHT;
+	}
+	y +=
+	  (si->staffspace + staff->space_below);
+      }
+    //itp.haslyrics = FALSE;
+    curstaff = curstaff->next;
+  }// for all the staffs
 
 
  
@@ -857,15 +834,14 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
 gint
 scorearea_expose_event (GtkWidget * widget, GdkEventExpose * event)
 {
-
 DenemoGUI *gui = Denemo.gui;
-  do{
-  /* Clear the backing pixmap */
-  if(Denemo.gui->input_source!=INPUTKEYBOARD &&
-     (Denemo.prefs.overlays || (Denemo.gui->input_source==INPUTAUDIO))
-     && pitch_entry_active(gui)) {
-    gdk_draw_rectangle (gui->pixmap,
-			gcs_lightbluegc(),
+ do{
+   /* Clear the backing pixmap */
+   if(Denemo.gui->input_source!=INPUTKEYBOARD &&
+      (Denemo.prefs.overlays || (Denemo.gui->input_source==INPUTAUDIO))
+      && pitch_entry_active(gui)) {
+     gdk_draw_rectangle (gui->pixmap,
+			 gcs_lightbluegc(),
 			TRUE,
 			0, 0,
 			widget->allocation.width, widget->allocation.height);
@@ -901,20 +877,3 @@ DenemoGUI *gui = Denemo.gui;
 }
 
 
-
-
-/**
- * create an editable display of the passed str at the bottom 
- * of the drawing area 
- */
- /*
-    void
-    display_string (gchar * str, DenemoGUI *gui)
-    {
-
-    if (GTK_WIDGET_VISIBLE (gui->scorearea))     not in toplevel 
-    set_text_node (NULL, gui);
-
-
-    }
-  */
