@@ -24,6 +24,7 @@
 #include "scoreops.h"
 #include "objops.h"
 #include "xmldefs.h"
+#include "lyric.h"
 
 #define ENTER_NOTIFY_EVENT "focus-in-event"
 #define LEAVE_NOTIFY_EVENT "focus-out-event"
@@ -1324,7 +1325,7 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
   gint objnum;//count of objects in measure starting at 1
   gint open_braces;//Keep track of the number of open brace "{" chars in the music, in case of imbalance.
   GString *str = g_string_new("");
-  GString * lyrics = g_string_new("");
+  GList * lyrics = NULL;
   GString * figures = g_string_new("");
   GString * fakechords = g_string_new("");
   prevduration = 0;
@@ -1344,10 +1345,14 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
   
   /* a button and mark for the lyrics of this staff */
   GString *lyrics_name = g_string_new(movement);
-  if(curstaffstruct->haslyrics) {
+  if(curstaffstruct->verses) {
     g_string_prepend(lyrics_name, "Lyrics for ");
     g_string_append_printf(lyrics_name, " Voice %d", voice_count);
     insert_music_section(gui, lyrics_name->str);
+    GList *g;
+    for(g=curstaffstruct->verses;g;g=g->next) {
+      lyrics = g_list_append(lyrics, get_text_from_view(g->data));
+    }
   }
 
   /* a button and mark for the figures of this staff */
@@ -1476,13 +1481,6 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
 	    empty_measure=FALSE; 
 
 
-
-
-	  if( (curobj->type==LILYDIRECTIVE)  &&
-	      (((lilydirective *) curobj->object)->postfix->len==0) &&
-	      (((lilydirective *) curobj->object)->prefix->len==0))
-	    continue;
-
  if(curobj->type==LILYDIRECTIVE){
 #define OUTPUT_LILY(what) \
   if(((lilydirective *) curobj->object)->what && ((lilydirective *) curobj->object)->what->len) {\
@@ -1575,18 +1573,17 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
 
 	  if(curobjnode) {
 	    curobj = (DenemoObject *) curobjnode->data;
-	    /* lyrics, figures and chord symbols */
+	    /*  figures and chord symbols */
 	    if(curobj->type==CHORD) {
 	      chord *pchord = (chord *) curobj->object;
-	      if(curstaffstruct->haslyrics && pchord->notes)
-		output_lyric(lyrics, pchord, &is_syllable, &center_lyric);
+
 	       
 	      if(curstaffstruct->hasfigures)
 		output_figured_bass (si, figures, pchord);
 	      
 	      if (curstaffstruct->hasfakechords)
 		output_fakechord(si, fakechords, pchord);
-	    /* end of lyrics, figures and chord symbols*/
+	    /* end of figures and chord symbols*/
 	    }
 	  }
 	  }//in obj range
@@ -1623,21 +1620,24 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
   gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, &iter, str->str, -1, INEDITABLE, invisibility,NULL);
   
 
-  if (lyrics->len)
+  if (lyrics)
     {  
-      GString *temp = g_string_new("");
-      gtk_text_buffer_get_iter_at_mark (gui->textbuffer, &iter, gtk_text_buffer_get_mark(gui->textbuffer, lyrics_name->str));
-      /* output lyrics prolog */
-      if(curstaffstruct->lyrics_prolog && curstaffstruct->lyrics_prolog->len) {
-	insert_editable(&curstaffstruct->lyrics_prolog, curstaffstruct->lyrics_prolog->str, &iter, invisibility, gui);
-      } else {	
-	g_string_printf(temp, "%s%sLyrics = \\lyricmode { \n", movement,
-			voice);
-	insert_editable(&curstaffstruct->lyrics_prolog, temp->str, &iter,  invisibility, gui);
+      GList *g;
+      gint versenum;
+      for(versenum=1, g=lyrics;g;g=g->next, versenum++) {
+	GString *versename = g_string_new("");
+	GString *temp = g_string_new("");
+	g_string_printf(temp, "Verse%d", versenum);
+	set_lily_name(temp, versename);
+	gtk_text_buffer_get_iter_at_mark (gui->textbuffer, &iter, gtk_text_buffer_get_mark(gui->textbuffer, lyrics_name->str));
+	g_string_printf(temp, "%s%sLyrics%s = \\lyricmode { \n", movement,
+			voice, versename->str);
+	g_string_append_printf(temp, "%s \n}\n", g->data);
+	gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, &iter, temp->str, -1, INEDITABLE, invisibility,NULL);
+	g_string_free(temp, TRUE);
+	g_string_free(versename, TRUE);
+	g_free(g->data);
       }
-      g_string_printf(temp, "%s \n}\n", lyrics->str);
-      gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, &iter, temp->str, -1, INEDITABLE, invisibility,NULL);
-      g_string_free(temp, TRUE);
     }
   g_string_free(lyrics_name, TRUE);
 
@@ -1680,7 +1680,7 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
   g_string_free(fakechords_name, TRUE);
 
   g_string_free(str, TRUE);
-  g_string_free(lyrics, TRUE);
+  //g_string_free(lyrics, TRUE);
   g_string_free(figures, TRUE);
   g_string_free(fakechords, TRUE);
 } /* outputStaff */
@@ -1705,11 +1705,14 @@ void merge_lily_strings (DenemoGUI *gui) {
     if(target) { 
       gchar *lily = get_text(gui, anchor);   
       if(strcmp(lily, g_object_get_data(G_OBJECT(anchor),ORIGINAL))){
-	//g_print("Compare %s\nwith %s for target %p\n", lily, g_object_get_data(anchor,ORIGINAL), *target);
+	//	g_print("Compare %s\nwith %s for target %p\n", lily, g_object_get_data(anchor,ORIGINAL), *target);
 	if(!*target)
 	  *target = g_string_new(lily);
 	else
 	  g_string_assign(*target, lily);
+
+
+	//this does not prevent corruption!!!!! on deleting all the string...
 	/* white space becomes empty string */
 	g_strstrip(lily);
 	if(*lily == '\0')
@@ -1717,12 +1720,15 @@ void merge_lily_strings (DenemoGUI *gui) {
 	    g_string_free(*target, TRUE);
 	    *target = g_string_new("");
 	  }
+#if 0
+	g_print("target %p at %p holds %s\n", *target, target, (*target)->str);
+#endif
+	/* this is    ((DenemoDirective*)((DenemoObject*)(Denemo.gui->si->currentobject->data))->object)->postfix */
 	g_free(g_object_get_data(G_OBJECT(anchor),ORIGINAL));
 	g_object_set_data(G_OBJECT(anchor),ORIGINAL, get_text(gui, anchor));
 	
 
-	gui->changecount++;
-	set_title_bar(gui);
+	score_status(gui, TRUE);
 	g_free(lily);
       }
 
@@ -1809,8 +1815,7 @@ void custom_lily_cb (GtkAction *action, gpointer param) {
     anchor = insert_scoreblock_section(gui, NULL, sb);
     gui->custom_scoreblocks = g_list_prepend(gui->custom_scoreblocks, sb);
     g_object_set_data(G_OBJECT(anchor),ORIGINAL, lily);
-    gui->changecount++;
-    set_title_bar(gui);
+    score_status(gui, TRUE);
     refresh_lily_cb(action, gui);
     GtkTextIter iter;//Place cursor at end of MUSIC which is where the score block has been placed
     gtk_text_buffer_get_iter_at_mark (gui->textbuffer, &iter, gtk_text_buffer_get_mark(gui->textbuffer, MUSIC));
@@ -1913,7 +1918,7 @@ output_score_to_buffer (DenemoGUI *gui, gboolean all_movements, gchar * partname
   /*figured basses */
 
   /*lyrics */
-  GString *lyrics = NULL;
+  //GString *lyrics = NULL;
 
   if(gui->textbuffer && (gui->changecount==gui->lilysync)
      && !strcmp(gui->namespec, namespec)) {
@@ -2137,12 +2142,23 @@ output_score_to_buffer (DenemoGUI *gui, gboolean all_movements, gchar * partname
 	      g_string_append_printf(staffdefinitions, TAB TAB"\\%s%sContext\n"TAB TAB"}\n", movement_name->str, voice_name->str);
 	      g_string_append_printf(scoreblock, "%s",thestr->str);
 
-	      if (curstaffstruct->haslyrics)
-	      {
-		g_string_append_printf(staffdefinitions, 
-				TAB TAB" \\lyricsto %s%s \\new Lyrics \\%s%sLyrics\n", 
+	      if (curstaffstruct->verses)
+		{
+		  GList *g;
+		  gint versenum;
+		  for(g=curstaffstruct->verses, versenum=1;g;g=g->next, versenum++) {
+		    GString *versename = g_string_new("");
+		    GString *temp = g_string_new("");
+		    g_string_printf(temp, "Verse%d", versenum);
+		    set_lily_name(temp, versename);
+		    g_string_append_printf(staffdefinitions, 
+				TAB TAB" \\lyricsto %s%s \\new Lyrics \\%s%sLyrics%s\n", 
 				voice_name->str, movement_name->str, 
-				movement_name->str, voice_name->str);
+				movement_name->str, voice_name->str, versename->str);
+		    g_string_free(versename, TRUE);
+		    g_string_free(temp, TRUE);
+		    
+		  }
 	      }
 
 	      if (curstaffstruct->hasfigures)
@@ -2448,7 +2464,7 @@ static lily_keypress(GtkWidget *w, GdkEventKey *event, DenemoGUI *gui) {
     return FALSE;
   // if you have a visible marker you do this gtk_text_iter_backward_cursor_position(&cursor);
   GtkTextChildAnchor *anchor = gtk_text_iter_get_child_anchor(&cursor);
-  // g_print("Got a keypress event at anchor %p\n", anchor);
+  //g_print("Got a keypress event at anchor %p\n", anchor);
   //g_print("The character is %x keyval %x at %d\n", (guint)gdk_keyval_to_unicode(event->keyval), event->keyval,  gtk_text_iter_get_line_offset(&cursor));
   if(anchor){
     gint objnum =  (gint) g_object_get_data(G_OBJECT(anchor), OBJECTNUM);
@@ -2481,7 +2497,7 @@ static lily_keypress(GtkWidget *w, GdkEventKey *event, DenemoGUI *gui) {
 	  g_object_set_data(G_OBJECT(anchor),ORIGINAL, get_text(gui, anchor));
 	  //g_print("prepended %s (%x)\n", key, *key);
 	}
-	gui->changecount++;
+	score_status(gui, TRUE);
 	refresh_lily_cb(NULL, gui);
 	gtk_text_buffer_get_iter_at_mark(gui->textbuffer, &cursor, gtk_text_buffer_get_insert(gui->textbuffer));
 	if(gtk_text_iter_forward_char (&cursor))
