@@ -54,6 +54,8 @@
 #include <stdio.h>
 #include <time.h>
 #include <ctype.h>
+#include <errno.h>
+#include "smf.h"
 
 #include "instrumentname.h"
 #include <denemo/denemo.h>
@@ -182,7 +184,7 @@ compress (int range, int invalue)
       fprintf (stderr, "compress %d %d\n", invalue, outvalue);
     }
 
-  return outvalue;
+  return outvalue & 0x7F;
 }
 
 /****************************************************************/
@@ -339,27 +341,32 @@ midi_delta (FILE * fd, long delta, int more)
 /**
  * this is used by several meta events
  */
-
-void
-midi_meta_text (FILE * fd, int metatype, char *string)
+static smf_event_t *
+midi_meta_text (int metatype, char *string)
 {
   int len;
-
-  midi_delta (fd, 0, 0);
-
+  smf_event_t *event = smf_event_new();
   len = strlen (string);
-
+  if(len>255) {
+    g_warning("Truncating string %s\n", string);
+    len = 255;
+  }
+  gchar *buffer = (gchar*)malloc(len+3);
+  event->midi_buffer = buffer;
+  event->midi_buffer_length = len+3;
   /* meta event */
-  midi_byte (fd, 0xff);
+  *buffer++ =  0xff;
 
   /* meta type */
-  midi_byte (fd, metatype);
+  *buffer++ =  metatype;
 
   /* meta size */
-  midi_byte (fd, len);
+  *buffer++ = (char)len;
 
   /* meta text */
-  fprintf (fd, string);
+  strncpy(buffer, string, len);
+  *(buffer+len) = 0;
+  return event; 
 }
 
 /**
@@ -369,6 +376,8 @@ midi_meta_text (FILE * fd, int metatype, char *string)
 void
 midi_comment (FILE * fd, char *txt1, long number, char *txt2)
 {
+#if 0
+  //disabled: size not checked
   char buf[200];
   if (number)
     {
@@ -380,6 +389,7 @@ midi_comment (FILE * fd, char *txt1, long number, char *txt2)
       sprintf (buf, "%s%s", txt1, txt2);
       midi_meta_text (fd, 1, buf);
     }
+#endif
 }
 
 /**
@@ -389,10 +399,12 @@ midi_comment (FILE * fd, char *txt1, long number, char *txt2)
 void
 midi_com2 (FILE * fd, char *txt1, long number1, long number2)
 {
+#if 0
   char buf[200];
 
   sprintf (buf, "%s %ld %ld", txt1, number1, number2);
   midi_meta_text (fd, 1, buf);
+#endif
 }
 
 /**
@@ -412,10 +424,16 @@ midi_channel_event (FILE * fd, int type, int chan, int note, int val)
  * almost only used for program change
  */
 
-void
-midi_change_event (FILE * fd, int type, int chan, int val)
+static smf_event_t *
+midi_change_event (int type, int chan, int val)
 {
-  midi_2_bytes (fd, type | chan, val);
+  smf_event_t *event = smf_event_new();
+  gchar *buffer = malloc(2);
+  event->midi_buffer = buffer;
+  event->midi_buffer_length = 2;
+  *buffer++ = type | chan;
+  *buffer++ = val;
+  return event;
 }
 
 /**
@@ -429,12 +447,16 @@ midi_end_of_track (FILE * fd)
 }
 
 /**
- * meta event: set time signature, time scale and metronome
+ * meta event: set time signature, time scale and metronome FF 58 04 nn dd cc bb Time Signature
  */
 
-void
-midi_timesig (FILE * fd, int upper, int lower)
+static smf_event_t *
+midi_timesig (int upper, int lower)
 {
+  smf_event_t *event = smf_event_new();
+  gchar *buffer = malloc(7);
+  event->midi_buffer = buffer;
+  event->midi_buffer_length = 7;
   div_t n;	
   int click = 24;
 
@@ -443,38 +465,53 @@ midi_timesig (FILE * fd, int upper, int lower)
     click = 24 * ((float) n.quot + (float) n.rem);
   }
 
-  midi_4_bytes (fd, 0, 0xff, 0x58, 4);
-  midi_4_bytes (fd, upper, twolog (lower), click, 8);
+  *buffer++ =0xff, *buffer++ =0x58, *buffer++ =4;
+  *buffer++ =upper, *buffer++ =twolog (lower), *buffer++ =click, *buffer++ =8;
+  return event;
 }
 
 /**
  * meta event: set key signature
  */
 
-void
-midi_keysig (FILE * fd, gint key, gint isminor)
+static smf_event_t *
+midi_keysig (gint key, gint isminor)
 {
-  midi_4_bytes (fd, 0, 0xff, 0x59, 2);
-  midi_2_bytes (fd, key, isminor);
+  smf_event_t *event = smf_event_new();
+  gchar *buffer = malloc(5);
+  event->midi_buffer = buffer;
+  event->midi_buffer_length = 5;
+  *buffer++ = 0xff;
+  *buffer++ = 0x59;
+  *buffer++ =2;
+  *buffer++ = key;
+  *buffer++ =isminor;
+  return event;
 }
 
 /**
  * meta event: set system clock speed
  */
 
-void
-midi_tempo (FILE * fd, long tempo)
+static smf_event_t *
+midi_tempo (long tempo)
 {
   long midi_tempo;
-
+  smf_event_t *event = smf_event_new();
+  gchar *buffer = malloc(6);
+  event->midi_buffer = buffer;
+  event->midi_buffer_length = 6;
   if (tempo == 0)
     {
       tempo = 1;
     }
-  midi_delta (fd, 0, 0);
-  midi_3_bytes (fd, 0xff, 0x51, 3);
+  *buffer++ = 0xff;
+  *buffer++ = 0x51;
+  *buffer++ =  3;
   midi_tempo = 60000000 / tempo;
-  midi_medium (fd, midi_tempo);
+  *buffer++ = (midi_tempo >> 16) & 255;
+  *buffer++ = (midi_tempo >> 8) & 255, (midi_tempo >> 0) & 255;
+return event;
 }
 
 /**
@@ -721,7 +758,7 @@ slur_update (int *state, int begin, int end)
 }
 
 /**
- * insert a note and it's properties in the table
+ * insert a note and its properties in the table
  */
 
 void
@@ -889,11 +926,131 @@ compute_beat (long ticks, long ticks_in_a_beat,
 
 #define percent(t,p)	((t*p)/100)
 
+
+/* puts an event into track if buffer contains valid midi message.
+   and frees buffer. Returns the event, or NULL if invalid buffer.
+ */
+static smf_event_t* put_event(gchar *buffer, gint numbytes, DenemoObject *curobj, smf_track_t *track) {
+  smf_event_t *event = NULL;
+  if(numbytes && is_status_byte(buffer[0]))
+    event = smf_event_new_from_pointer(buffer, numbytes);
+  if(event && smf_event_is_valid(event)) {
+    smf_track_add_event_delta_pulses(track, event, 0);
+    curobj->midi_events = g_list_append(curobj->midi_events, event);
+    //g_print("Added %x\n", event->midi_buffer[0]);
+  }
+  g_free(buffer);
+  return event;
+}
+
+static gint directive_get_midi_override(DenemoDirective *directive) {	
+  return directive->override&DENEMO_MIDI_MASK;
+}
+static gint directive_get_midi_interpretation(DenemoDirective *directive) {	
+  return directive->override&DENEMO_MIDI_INTERPRETATION_MASK;
+}
+static gint directive_get_midi_action(DenemoDirective *directive) {	
+  return directive->override&DENEMO_MIDI_ACTION_MASK;
+}
+
+static gchar *directive_get_midi_buffer(DenemoDirective *directive, gint *pnumbytes) {		    
+  if(directive->midibytes) {
+    gchar *bytes;
+    bytes = directive->midibytes->str;
+    //g_print("Got %s as midi bytes\n", bytes);
+    char *next;
+    char val;
+    gint i, numbytes;
+    errno = 0;
+    for(i=0, next=bytes;*next; next++){ 
+      val = strtol(next, &next, 0);
+#if 0
+      if(i==0 && val==0)
+	errno = EINVAL;
+#endif
+      if(errno) {
+	g_warning("Bytes %s bad format for MIDI\n", bytes);
+	return NULL;
+      }
+      i++;
+      if(*next==0)
+	break;
+    }
+    if(errno)
+      return NULL;
+    numbytes = i;
+    gchar *buffer = (gchar*) g_malloc0(numbytes);
+    for(i=0, next=bytes;i<numbytes;i++, next++){ 
+      buffer[i] = (char) strtol(next, &next, 0);
+      //g_print("byte %x\n", buffer[i]);
+    }
+    *pnumbytes = numbytes;
+    return buffer;
+  }
+  return NULL;
+}
+
+static gint directive_get_midi_val(DenemoDirective *directive) {
+  gint val = 0;		    
+  if(directive->midibytes) {
+    gchar *bytes;
+    bytes = directive->midibytes->str;
+    errno = 0;
+    val = strtol(bytes, NULL, 0);
+    if(errno) {
+      g_warning("String %s is bad format for MIDI value\n", bytes);
+      return 0;
+    }
+  }
+  return val;
+}
+
+/* change the volume according to the values passed in */
+static void 
+change_volume(gint *volume, gint midi_val, gint  midi_interpretation,gint  midi_action)
+{
+  gdouble val;
+  // g_print("midi_val %d cur_volume %d\n", midi_val, cur_volume);
+  val = (gdouble) midi_val;
+  if(midi_interpretation | DENEMO_OVERRIDE_PERCENT)
+    val = *volume * (midi_val/100.0);
+  if(midi_action == DENEMO_OVERRIDE_ONCE)
+    g_warning("Scripting error, ONCE for standalone directive is meaningless");
+  if(midi_action == DENEMO_OVERRIDE_RAMP)
+    g_warning("Not implemented ramp yet");
+  if(midi_action == DENEMO_OVERRIDE_STEP) {
+    if(midi_interpretation | DENEMO_OVERRIDE_RELATIVE)
+      *volume += val;
+    else
+      *volume = val;
+  }  
+  if(*volume>127) *volume=127;
+  if(*volume<0) *volume=0;
+}
+/* change the tempo according to the values passed in */
+static void 
+change_tempo(gint *tempo, gint midi_val, gint  midi_interpretation,gint  midi_action)
+{
+  gdouble val;
+  val = (gdouble) midi_val;
+  if(midi_interpretation | DENEMO_OVERRIDE_PERCENT)
+    val = *tempo * (midi_val/100.0);
+  if(midi_action == DENEMO_OVERRIDE_ONCE)
+    g_warning("Not implemented change of tempo for one chord");
+  if(midi_action == DENEMO_OVERRIDE_RAMP)
+    g_warning("Not implemented ramp yet");
+  if(midi_action == DENEMO_OVERRIDE_STEP) {
+    if(midi_interpretation | DENEMO_OVERRIDE_RELATIVE)
+      *tempo += val;
+    else
+      *tempo = val;
+  }
+}
+  
 /*
  * the main midi output system (somewhat large)
  */
 
-
 /****************************************************************/
 /****************************************************************/
 /****************************************************************/
@@ -906,7 +1063,7 @@ gdouble
 exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 {
   /* variables for reading and decoding the object list */
-  FILE *fd;
+  smf_event_t *event = NULL;
   staffnode *curstaff;
   DenemoStaff *curstaffstruct;
   measurenode *curmeasure;
@@ -919,7 +1076,7 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
   gint mid_c_offset;
   GList *curtone;
   gdouble fraction;
-  GString *filename = g_string_new (thefilename);
+
   gint measurenum, last = 0;
 
   /* variables for generating music */
@@ -931,7 +1088,7 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
   long ticks_at_bar = 0;
   int cur_volume;
   gboolean mute_volume;
-  int mix;
+
   int midi_channel = (-1);
   int tracknumber = 0;
   int timesigupper = 4;
@@ -996,34 +1153,8 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
   /* just curious */
   time (&starttime);
 
-  /* Append .mid to the filename */
-  /*g_string_append (filename, ".midi"); */
-
-  /* Now open the file */
-  fd = fopen (filename->str, "w");
-
-  /* verify file descriptor */
-  if (!fd)
-    {
-      perror (filename->str);
-      fprintf (stderr, "Can't open midi file %s for writing!\n",
-	       filename->str);
-      return;
-    }
-
-  /* MIDI file header */
-  midi_4_bytes (fd, 'M', 'T', 'h', 'd');
-  midi_long (fd, 6);
-
-  /* midi file type */
-  midi_short (fd, 1);
-
-  /* tracks in the file */
-  /* this gets overwritten before closing the file */
-  midi_short (fd, 0);
-
-  /* ticks in a quarter note */
-  midi_short (fd, MIDI_RESOLUTION);
+  smf_t *smf = smf_new();
+  int dummy1 = smf_set_ppqn(smf, MIDI_RESOLUTION);
 
 /*
  * end of headers and meta events, now for some real actions
@@ -1047,6 +1178,8 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 
       /* select a suitable track number */
       tracknumber++;
+      smf_track_t *track = smf_track_new();
+      smf_add_track(smf, track);
       if (!strcmp (curstaffstruct->midi_instrument->str, "drums"))
 	{
 	  midi_channel = 9;
@@ -1056,21 +1189,19 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 	  midi_channel = (tracknumber >= 9) ? tracknumber + 1 : tracknumber;
 	}
 
-      /* MIDI track header */
-      midi_4_bytes (fd, 'M', 'T', 'r', 'k');
-
-      /* remember track size (to be filled in later, when we know the size) */
-      midi_long (fd, 0);
-      track_start_pos[tracknumber] = ftell (fd);
 
       /* track name */
-      midi_meta_text (fd, 3, curstaffstruct->lily_name->str);
+      event = midi_meta_text (3, curstaffstruct->lily_name->str);
+      smf_track_add_event_delta_pulses(track, event, 0);
 
       /* tempo */
-      midi_tempo (fd, si->tempo);
+      gint cur_tempo = si->tempo;
+      event = midi_tempo (cur_tempo);
+      smf_track_add_event_delta_pulses(track, event, 0);
 
       /* The midi instrument */
-      midi_meta_text (fd, 4, curstaffstruct->midi_instrument->str);
+      event = midi_meta_text (4, curstaffstruct->midi_instrument->str);
+      smf_track_add_event_delta_pulses(track, event, 0);
 
       if (curstaffstruct->midi_prognum_override != TRUE){
       /* drums is a special case, else try to match an instrument name */
@@ -1089,11 +1220,14 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
       }
 
       /* set selected midi program */
-      midi_delta (fd, 0, 0);
-      midi_change_event (fd, MIDI_PROG_CHANGE, midi_channel, prognum);
+      g_print("Using channel %d\n", midi_channel);
+      event = midi_change_event (MIDI_PROG_CHANGE, midi_channel, prognum);
+      smf_track_add_event_delta_pulses(track, event, 0);
 
       /*key signature */
-      midi_keysig (fd, curstaffstruct->keysig.number, curstaffstruct->keysig.isminor);
+
+      event = midi_keysig (curstaffstruct->keysig.number, curstaffstruct->keysig.isminor);
+      smf_track_add_event_delta_pulses(track, event, 0);
 
       /* Time signature */
       timesigupper = curstaffstruct->timesig.time1;
@@ -1102,12 +1236,18 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
       timesiglower = curstaffstruct->timesig.time2;
       //printf("\nstime2 = %i\n", timesiglower);
 
-      midi_timesig (fd, timesigupper, timesiglower);
+       event = midi_timesig (timesigupper, timesiglower);
+       smf_track_add_event_delta_pulses(track, event, 0);
 
       /* set a default velocity value */
       cur_volume = curstaffstruct->volume;
       /* mute output if set */
       mute_volume = curstaffstruct->mute_volume;
+      if(cur_volume==0 && mute_volume==TRUE) {
+	g_warning("volume set to zero but not muted\nResetting volume\n");
+	cur_volume = 65;
+      }
+	
 
       /* reset measure */
       curmeasurenum = 0;
@@ -1142,12 +1282,14 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 	  measure_has_odd_tuplet = 0;
 	  ticks_at_bar = ticks_read;
 
-	  /* iterate for over objects in measure */
+	  /* iterate over objects in measure */
 	  for (curobjnode = (objnode *) curmeasure->data; curobjnode;
 	       curobjnode = curobjnode->next)
 	    {
 	      curobj = (DenemoObject *) curobjnode->data;
-
+	      if(curobj->midi_events)
+		g_list_free(curobj->midi_events);//data belongs to libsmf
+	      curobj->midi_events = NULL;
 	/*******************************************
 	 *	huge switch:
 	 * 	here we handle every kind of object
@@ -1162,15 +1304,49 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 	  /******************** 
 	   * one or more notes
 	   ********************/
-
-
+		  
+		 
 		  measure_is_empty = 0;
 		  if (debug)
 		    fprintf (stderr,
 			     "=============================== chord at %s\n",
 			     fmt_ticks (ticks_read));
 		  chordval = *(chord *) curobj->object;
-
+		  if(chordval.directives) {
+		        GList *g;
+			DenemoDirective *directive = NULL;
+			for(g=chordval.directives;g;g=g->next){
+			  gint numbytes;
+			  directive = (DenemoDirective *)g->data;
+			  gchar *buffer = directive_get_midi_buffer(directive, &numbytes);
+			  gint midi_override = directive_get_midi_override(directive);
+			  gint midi_interpretation = directive_get_midi_interpretation(directive);
+			  gint midi_action = directive_get_midi_action(directive);
+			  gint midi_val = directive_get_midi_val(directive);
+			  /* handle all types of MIDI overrides attached to chord here */
+			  switch(midi_override) 
+			    {
+			    case DENEMO_OVERRIDE_VOLUME:
+				change_volume(&cur_volume, midi_val, midi_interpretation, midi_action);
+			      break;
+			    case DENEMO_OVERRIDE_TEMPO:
+			      change_tempo(&cur_tempo, midi_val, midi_interpretation, midi_action);
+			      if(cur_tempo) {
+				event = midi_tempo (cur_tempo);
+				smf_track_add_event_delta_pulses(track, event, 0);
+			      } else
+				g_warning("Tempo change to 0 bpm is illegal");
+			      break;
+			      //etc			      
+			    default:
+			      if(buffer) 
+				if(NULL==put_event(buffer, numbytes, curobj, track))
+				  g_warning("Invalid midi bytes in chord directive\n");
+			      break;
+			    }
+			}//for each directive attached to the chord
+		  }//if there are directives
+		  /* FIXME sound grace notes either simultaneously for shorter duration or steal time .... */
 		  if(chordval.is_grace)
 		    break;
 
@@ -1287,6 +1463,9 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			  //	  curstaffstruct->transposition);
 			  notenumber += curstaffstruct->transposition;
 			  //printf ("notenumber = %i\n", notenumber);
+			  if(notenumber>127) {
+			    g_warning("Note out of range\n", notenumber = 60);
+			  }
 			  slur_note (note_status,
 				     slur_status,
 				     notenumber,
@@ -1312,39 +1491,44 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		      /* write delta */
 		      for (n = 0; n < 128; n++)
 			{
+			  gint mididelta;
 			  if (slur_on_p (note_status, n)
 			      || slur_kill_p (note_status, n))
 			    {
 			      if (notes_in_chord++ == 0)
 				{
-				  midi_delta (fd, ticks_read - ticks_written,
-					      0);
+				  mididelta = ticks_read - ticks_written;
 				  ticks_written = ticks_read;
 				}
 			      else
 				{
-				  midi_delta (fd, 0, 0);
+				  mididelta = 0;
 				}
 			    }
 
 			  /* compute velocity delta */
 			  rand_delta = i_random (&rand_sigma, vel_randfact);
-
 			  /* write note on/off */
 			  if (slur_on_p (note_status, n))
 			    {
-			      mix = compress(128, cur_volume + rand_delta + beat);
-			      midi_channel_event (fd, MIDI_NOTE_ON,
-						  midi_channel, n,
-						  (mute_volume ? 0:mix));
+			      // int mix = cur_volume? compress(128, cur_volume + rand_delta + beat) : 0;
+			      // FIXME the function compress is returning large values.
+			      event = smf_event_new_from_bytes (MIDI_NOTE_ON | midi_channel, n, (mute_volume ? 0:cur_volume/*FIXME as above, mix*/));
+			      smf_track_add_event_delta_pulses(track, event, mididelta);
+			      //g_print("Note on %x %x %x\n", MIDI_NOTE_ON | midi_channel, n, (mute_volume ? 0:cur_volume/*FIXME as above, mix*/));
+			      //g_print("event on %f\n", event->time_seconds);
+			      curobj->midi_events = g_list_append(curobj->midi_events, event);
 #if DEBUG
-			      printf ("volume = %i\n", (mute_volume ? 0:mix));
+			      g_print("'%d len %d'", event->event_number, event->midi_buffer_length);
+			      //printf ("volume = %i\n", (mute_volume ? 0:mix));
 #endif
 			    }
 			  else if (slur_kill_p (note_status, n))
-			    {
-			      midi_channel_event (fd, MIDI_NOTE_OFF,
-						  midi_channel, n, 12820);
+			    {// FIXME what is this 12820??????
+			      event = smf_event_new_from_bytes ( MIDI_NOTE_OFF | midi_channel, n, 0);
+			      //g_print("{%d}", event->event_number);
+			      smf_track_add_event_delta_pulses(track, event, mididelta);
+			      curobj->midi_events = g_list_append(curobj->midi_events, event);
 			    }
 			}
 		      /* end of first chord output loop */
@@ -1369,10 +1553,11 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			{
 			  if (slur_off_p (note_status, n))
 			    {
+			      gint mididelta;
 			      if (notes_in_chord++ == 0)
 				{
 				  width += ticks_read - ticks_written;
-				  midi_delta (fd, duration + width, 0);
+				  mididelta = duration + width;
 				  ticks_written += duration + width;
 				  if (ticks_written > ticks_read + duration)
 				    {
@@ -1382,17 +1567,17 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 				}
 			      else
 				{
-				  midi_delta (fd, 0, 0);
+				  mididelta = 0;
 				}
-
 			      /* write note off */
-			      midi_channel_event (fd, MIDI_NOTE_OFF,
-						  midi_channel, n, 60);
+			      event = smf_event_new_from_bytes ( MIDI_NOTE_OFF | midi_channel, n, 60);
+			      smf_track_add_event_delta_pulses(track, event, mididelta);
+			      curobj->midi_events = g_list_append(curobj->midi_events, event);
 			    }
 			}
 		      /* end of second chord output loop */
 
-		    }
+		    }//end of for notes in chord. Note that rests have no MIDI representation, of course.
 		  width = 0;
 #if slurdebug
 		  print_slurs (stderr,
@@ -1432,8 +1617,9 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		    }
 		  printf ("\nchange to timesigupper = %i\n", timesigupper);
 		  printf ("\nchange to timesiglower = %i\n", timesiglower);
-
-		  midi_timesig (fd, timesigupper, timesiglower);
+		  event = midi_timesig (timesigupper, timesiglower);
+		  smf_track_add_event_delta_pulses(track, event, 0);
+		  curobj->midi_events = g_list_append(curobj->midi_events, event);
 		  break;
 
 		case TUPOPEN:
@@ -1507,8 +1693,11 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		  // curobj->object
 		  //  ((keysig *) theobj->object)->number; referenced in src/measureops.cpp       
 		  //printf("\nKEYSIG type = %d\n", ((keysig *) curobj->object)->number);
-		  midi_keysig (fd, (((keysig *) curobj->object)->number),
+		  event = midi_keysig ((((keysig *) curobj->object)->number),
 			       curstaffstruct->keysig.isminor);
+		  smf_track_add_event_delta_pulses(track, event, 0);
+		  curobj->midi_events = g_list_append(curobj->midi_events, event);
+		  break;
 
 		case CLEF:
 
@@ -1517,8 +1706,42 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 	   ***********/
 
 		  break;
+
+		case LILYDIRECTIVE:
+		  {
+		    gint numbytes;
+		    gchar *buffer = directive_get_midi_buffer(curobj->object, &numbytes);
+		    gint midi_override = directive_get_midi_override(curobj->object);
+		    gint midi_interpretation = directive_get_midi_interpretation(curobj->object);
+		    gint midi_action = directive_get_midi_action(curobj->object);
+		    gint midi_val = directive_get_midi_val(curobj->object);
+		    switch(midi_override) 
+		      {
+		      case  DENEMO_OVERRIDE_VOLUME:
+			change_volume(&cur_volume, midi_val, midi_interpretation, midi_action);
+			break;
+		      case DENEMO_OVERRIDE_TEMPO:
+			change_tempo(&cur_tempo, midi_val, midi_interpretation, midi_action);
+			if(cur_tempo) {
+			  event = midi_tempo (cur_tempo);
+			  smf_track_add_event_delta_pulses(track, event, 0);
+			} else
+			  g_warning("Tempo change to 0 bpm is illegal");
+			break;
+		      case DENEMO_OVERRIDE_DURATION:
+			g_warning("Not implemented");
+			break;
+		      default:
+			if(buffer) {
+			  if(NULL==put_event(buffer, numbytes, curobj, track))
+			    g_warning("Directive has invalid MIDI bytes\n");
+			}
+			break;
+		      }
+		  }
+		  break;
 		default:
-		  fprintf (stderr, "ignoring type %d\n", curobj->type);
+		  fprintf (stderr, "midi ignoring type %d\n", curobj->type);
 		  break;
 		}
 	    }
@@ -1532,8 +1755,8 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 	    {
 	      if ((!measure_is_empty) && curmeasure->next)
 		{
-		  fprintf (stderr,
-			   "warning: ticks_read reset in %s "
+		  g_warning(
+			   "warning: overfull measure in %s "
 			   "measure %d from %ld to %ld "
 			   "\n%sdifference is %ld, measure began at %ld)\n",
 			   curstaffstruct->lily_name->str,
@@ -1553,11 +1776,11 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		  printf("\ninternal ticks = %d\n", internaltoticks(0));
 		}
 
-	      ticks_read = ticks_at_bar + measurewidth;//+ internaltoticks (0);
+		//ticks_read = ticks_at_bar + measurewidth;//+ internaltoticks (0);
 	    }
 	  else
 	    {
-	      fprintf (stderr, "[%d]", measurenum);
+	      ;//fprintf (stderr, "[%d]", measurenum);
 	    }
 	  fflush (stdout);
 
@@ -1566,22 +1789,13 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
        *************************/
 
 
-	  /* end the track (required!) */
-	  if ((!curmeasure->next)
-	      || (MAX (start, 1) + curmeasurenum == last + 1))
-	    {
-	      midi_end_of_track (fd);
-	      printf ("\nend of track\n");
-	    }
-	  track_end_pos[tracknumber] = ftell (fd);
-
 	}			/* Done with this staff */
 
     /***********************
      * Done with this track
      ***********************/
 
-      fprintf (stderr, "[%s done]\n", curstaffstruct->lily_name->str);
+      //fprintf (stderr, "[%s done]\n", curstaffstruct->lily_name->str);
       fflush (stdout);
       /*gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(si->progressbar), fraction + 
          gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(si->progressbar))); */
@@ -1591,47 +1805,27 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 	break;
     }
 
-  /*********************
-   * Insert track sizes
-   *********************/
-
-  /* insert track count in file header */
-  fseek (fd, 10, SEEK_SET);
-  midi_short (fd, tracknumber);
-  fflush (fd);
-
-  /* insert track byte count in track headers */
-  for (i = 1; i <= tracknumber; i++)
-    {
-      fseek (fd, track_start_pos[i] - 4, SEEK_SET);
-      midi_long (fd, track_end_pos[i] - track_start_pos[i]);
-      fflush (fd);
-
-      if (debug)
-	fprintf (stderr,
-		 "track %d from: %ld to %ld (%ld at %ld)\n", i,
-		 track_start_pos[i], track_end_pos[i],
-		 (track_end_pos[i] - track_start_pos[i]), ftell (fd) - 4);
-    }
-
   /********
    * Done!
    ********/
-
+  if(thefilename) {
+    int dummy2;
+    dummy2 = smf_save(smf, (const char*)thefilename);
+  }
   /* we are done */
-  fclose (fd);
 
-  /* just curious again
-  time (&endtime);
-  fprintf (stderr,
-	   "exportmidi %s done:\nfile: %s %s %s\ntotal elapsed time is %ld\n",
-	   EXPORTMIDI_VERSION, filename->str, __DATE__, __TIME__,
-	   endtime - starttime);
- */
-  //g_print("total duration %f seconds\n", 60.0*ticks_read/(MIDI_RESOLUTION*(gdouble)si->tempo));
+  if(si->smf) {
+    smf_t *temp = si->smf;
+    si->smf = NULL;
+    smf_delete(temp);
+  }
+  si->smf = smf;
 
-  g_string_free (filename, FALSE);
-  return  60.0*ticks_read/(MIDI_RESOLUTION*(double)si->tempo);
+  si->smfsync = si->changecount;
+
+  if(start || end)
+    return exportmidi(NULL, si, 0, 0);// recurse if we have not generated all the MIDI for si
+  return smf_get_length_seconds(smf);
 }
 
 /*
