@@ -17,6 +17,7 @@
 #include "xmldefs.h"
 #include "articulations.h"
 #include "view.h"
+#include "lilydirectives.h"
 #include <string.h>
 
 
@@ -227,21 +228,26 @@ LOOKUP(GROUP_END_STRING,DENEMO_GROUP_END)
     }
 }
 
-static gint
-parseDirective (xmlNodePtr parentElem, xmlNsPtr ns,
-	       DenemoDirective *directive)
-{
-  xmlNodePtr childElem;
+
+
 #define DO_DIREC(field) if (ELEM_NAME_EQ (childElem, #field))\
          directive->field = g_string_new(xmlNodeListGetString (childElem->doc,\
 						  childElem->xmlChildrenNode, 1));
 #define DO_INTDIREC(field) if (ELEM_NAME_EQ (childElem, #field))\
          directive->field = getXMLIntChild(childElem);
+
+static gint
+parseDirective (xmlNodePtr parentElem, xmlNsPtr ns,
+	       DenemoDirective *directive)
+{
+  xmlNodePtr childElem;
   FOREACH_CHILD_ELEM (childElem, parentElem) {
     DO_DIREC(tag);
     DO_DIREC(prefix);
     DO_DIREC(postfix);
     DO_DIREC(display); 
+    DO_DIREC(midibytes);
+    DO_INTDIREC(override);
     DO_INTDIREC(minpixels);
     DO_INTDIREC(x);
     DO_INTDIREC(y);
@@ -249,17 +255,95 @@ parseDirective (xmlNodePtr parentElem, xmlNsPtr ns,
     DO_INTDIREC(ty);
     DO_INTDIREC(gx);
     DO_INTDIREC(gy);
-    DO_INTDIREC(override);
+
     if(ELEM_NAME_EQ (childElem, "graphic_name")) {
       directive->graphic_name =  g_string_new(xmlNodeListGetString (childElem->doc,\
 						  childElem->xmlChildrenNode, 1));
       loadGraphicItem(directive->graphic_name->str, &directive->graphic,  &directive->width, &directive->height);
       /* FIXME,handle not loaded */
-    }
-      
+    }      
   }
+}
+
+static gint
+parseWidgetDirective (xmlNodePtr parentElem, xmlNsPtr ns, gboolean *fn())
+{
+  DenemoDirective *directive = NULL;
+  xmlNodePtr childElem;
+  gchar *graphic_name = NULL, *tag = NULL, *display = NULL;
+  FOREACH_CHILD_ELEM (childElem, parentElem) {
+    if(ELEM_NAME_EQ (childElem, "graphic_name")) {
+      graphic_name = xmlNodeListGetString (childElem->doc,
+						  childElem->xmlChildrenNode, 1);
+
+  }
+    if(ELEM_NAME_EQ (childElem, "tag")) {
+      tag =  xmlNodeListGetString (childElem->doc,
+						  childElem->xmlChildrenNode, 1);
+  }
+    if(ELEM_NAME_EQ (childElem, "display")) {
+      display =  xmlNodeListGetString (childElem->doc,
+						  childElem->xmlChildrenNode, 1);
+  }
+  }
+  directive = fn(tag, graphic_name?graphic_name:display);
+  if(directive==NULL)
+    return;
+  FOREACH_CHILD_ELEM (childElem, parentElem) {
+    DO_DIREC(tag);
+    DO_DIREC(prefix);
+    DO_DIREC(postfix);
+    DO_DIREC(display);
+
+    DO_DIREC(midibytes);
+    DO_INTDIREC(override);
+    DO_INTDIREC(minpixels);
+    DO_INTDIREC(x);
+    DO_INTDIREC(y);
+    DO_INTDIREC(tx);
+    DO_INTDIREC(ty);
+    DO_INTDIREC(gx);
+    DO_INTDIREC(gy);
+  }
+
+ 
+#if  0
+  FOREACH_CHILD_ELEM (childElem, parentElem) {
+    if(ELEM_NAME_EQ (childElem, "graphic_name")) {
+      directive->graphic_name =  g_string_new(xmlNodeListGetString (childElem->doc,
+						  childElem->xmlChildrenNode, 1));
+      directive = fn(directive->tag->str, directive->graphic_name->str );
+    }      
+  }
+#endif
+}
+
+
 #undef DO_DIREC 
 #undef DO_INTDIREC  
+
+static void
+parseVerse (xmlNodePtr parentElem, xmlNsPtr ns,
+	      GtkWidget *verse)
+{
+  gchar*   text = xmlNodeListGetString (parentElem->doc, parentElem->xmlChildrenNode, 1);
+
+  gtk_text_buffer_set_text (gtk_text_view_get_buffer (verse), text, -1);
+  //gtk_text_buffer_set_modified(gtk_text_view_get_buffer(verse), FALSE);
+  g_signal_connect (G_OBJECT (gtk_text_view_get_buffer (verse)), "changed", G_CALLBACK (lyric_change), NULL);
+  g_free (text);
+}
+
+static GList *
+parseVerses (DenemoScore *si, DenemoStaff *staff, xmlNodePtr parentElem, xmlNsPtr ns)
+{
+  GList *verses = NULL;
+  xmlNodePtr childElem;
+  FOREACH_CHILD_ELEM (childElem, parentElem) {
+    GtkWidget *verse = add_verse_to_staff(si, staff);
+    parseVerse(childElem, ns, verse);
+  }
+  return verses;
 }
 
 static GList *
@@ -274,6 +358,16 @@ parseDirectives (xmlNodePtr parentElem, xmlNsPtr ns)
   }
   return directives;
 }
+
+static void
+parseWidgetDirectives (xmlNodePtr parentElem, xmlNsPtr ns, gboolean *fn())
+{
+  GList *directives = NULL;
+  xmlNodePtr childElem;
+  FOREACH_CHILD_ELEM (childElem, parentElem) {
+    parseWidgetDirective(childElem, ns, fn);
+  }
+}
 /**
  * Return the numerator and denominator from the given XML fraction.
  */
@@ -594,32 +688,18 @@ parseTimeSignature (xmlNodePtr timeSigElem, xmlNsPtr ns, timesig* timesig)
  *
  *
  */
+static GString *Lyric = NULL;
+
 static void
 parseLyric (xmlNodePtr lyricElem, DenemoObject * curobj)
 {
   gchar *lyric = (gchar *) xmlNodeListGetString (lyricElem->doc,
 						 lyricElem->xmlChildrenNode,
 						 1);
-
-  gchar *extend = (gchar *) xmlGetProp (lyricElem, (xmlChar *) "extend");
-  gchar *center = (gchar *) xmlGetProp (lyricElem, (xmlChar *) "center");
-
-  ((chord *) curobj->object)->lyric = g_string_new (lyric);
-
-  if (!strcmp (extend, "true"))
-    ((chord *) curobj->object)->is_syllable = TRUE;
-  else
-    ((chord *) curobj->object)->is_syllable = FALSE;
-
-  if (!strcmp (center, "true"))
-    ((chord *) curobj->object)->center_lyric = TRUE;
-  else
-    ((chord *) curobj->object)->center_lyric = FALSE;
-
+  if(lyric)
+    Lyric = g_string_append(Lyric, lyric);
+  Lyric = g_string_append(Lyric, " ");
   g_free (lyric);
-  g_free (extend);
-  g_free (center);
-
 }
 
 
@@ -1347,7 +1427,8 @@ parseChord (xmlNodePtr chordElem, xmlNsPtr ns,
 	  }
     DO_DIREC(prefix)
     DO_DIREC(postfix)
-    DO_DIREC(display)  
+    DO_DIREC(display) 
+    DO_DIREC(midibytes)  
 #undef DO_DIREC
 	// End 0.8.2 support
 	else if (ELEM_NAME_EQ (childElem, "chordize"))
@@ -1403,6 +1484,7 @@ parseLilyDir (xmlNodePtr LilyDirectiveElem, xmlNsPtr ns, DenemoScore *si)
 
   GET_STR_FIELD(tag);
   GET_STR_FIELD(display);
+  GET_STR_FIELD(midibytes);
   GET_STR_FIELD(graphic_name);
   GET_STR_FIELD(prefix);
   if(thedirective->graphic_name && thedirective->graphic_name->len)
@@ -1615,7 +1697,7 @@ parseSetupInfo (xmlNodePtr editInfoElem, xmlNsPtr ns, DenemoGUI * gui)
 	  }
 	else if (ELEM_NAME_EQ (childElem, "score-directives"))
 	  {
-	    gui->lilycontrol.directives = parseDirectives(childElem, ns);
+	    /*gui->lilycontrol.directives =*/ parseWidgetDirectives(childElem, ns, score_directive_put_graphic);
 	  }
 
 
@@ -1909,9 +1991,14 @@ parseStaff (xmlNodePtr staffElem, xmlNsPtr ns, DenemoScore * si)
 	  }
 	else if (ELEM_NAME_EQ (childElem, "haslyrics"))
 	  {
-	    curStaff->haslyrics = getXMLIntChild (childElem);
+	    //backward compatibility only
 	    
 	  }
+	else if (ELEM_NAME_EQ (childElem, "verses"))
+	  {
+	    
+	    parseVerses(si, curStaff, childElem, ns);
+	  }
 	else if (ELEM_NAME_EQ (childElem, "instrument"))
 	  {
 	    gchar *temp = (gchar *) xmlNodeListGetString
@@ -1968,11 +2055,11 @@ parseStaff (xmlNodePtr staffElem, xmlNsPtr ns, DenemoScore * si)
 	  }
 	else if (ELEM_NAME_EQ (childElem, "staff-directives"))
 	  {
-	    curStaff->staff_directives = parseDirectives(childElem, ns);
+	    /*curStaff->staff_directives =*/ parseWidgetDirectives(childElem, ns, staff_directive_put_graphic);
 	  }
 	else if (ELEM_NAME_EQ (childElem, "voice-directives"))
 	  {
-	    curStaff->voice_directives = parseDirectives(childElem, ns);
+	    /*curStaff->voice_directives =*/ parseWidgetDirectives(childElem, ns, voice_directive_put_graphic);
 	  }
 	else if (ELEM_NAME_EQ (childElem, "clef-directives"))
 	  {
@@ -2132,10 +2219,12 @@ parseInitVoiceParams (xmlNodePtr initVoiceParamsElem, xmlNsPtr ns,
 			g_warning ("<staff-ref> points to a <%s>, not a "
 				   "<staff>", staffElem->name);
 		      }
-
+		    
 		    si->currentprimarystaff = si->currentstaff;
+		    
 		    if (parseStaff (staffElem, ns, si) != 0)
 		      return -1;
+		   	      
 		  }
 
 		sPrevStaffElem = staffElem;
@@ -2252,10 +2341,8 @@ parseMeasures (xmlNodePtr measuresElem, xmlNsPtr ns, DenemoScore * si)
 				       &slurEndChordElems,
 				       &crescEndChordElems,
 				       &diminEndChordElems, &notesElem);
-		  /* old format files will not have haslyrics... fields of staff explicit
+		  /* old format files will not have has... fields of staff explicit
 		   so for backwards compatibility we reconstruct it here*/
-		  if(((chord *) curObj->object)->lyric)
-		    ((DenemoStaff *) si->currentstaff->data)->haslyrics = TRUE;
 		  if(((chord *) curObj->object)->figure)
 		    ((DenemoStaff *) si->currentstaff->data)->hasfigures = TRUE;
 		  if(((chord *) curObj->object)->fakechord)
@@ -2455,7 +2542,8 @@ parseVoice (xmlNodePtr voiceElem, xmlNsPtr ns, DenemoGUI * gui)
   newstaff (gui, ADDFROMLOAD, DENEMO_NONE);
   si->currentstaff = g_list_last (si->thescore);
   si->currentmeasurenum = 1;
-
+ Lyric = g_string_new("");
+		 
   /* Parse the child elements. */
 
   childElem = getXMLChild (voiceElem, "voice-info", ns);
@@ -2472,7 +2560,16 @@ parseVoice (xmlNodePtr voiceElem, xmlNsPtr ns, DenemoGUI * gui)
   RETURN_IF_ELEM_NOT_FOUND ("voice", childElem, "measures");
   if (parseMeasures (childElem, ns, si) != 0)
     return -1;
-
+  if(Lyric->len) {	     
+    DenemoStaff *staff = (DenemoStaff *) si->currentstaff->data;
+    add_verse_to_staff(si, staff);
+    gtk_text_buffer_set_text (gtk_text_view_get_buffer (staff->currentverse->data), Lyric->str, Lyric->len);
+    g_signal_connect (G_OBJECT (gtk_text_view_get_buffer (staff->currentverse->data)), "changed", G_CALLBACK (lyric_change), NULL);
+    //allow save on backward compatibility files... gtk_text_buffer_set_modified(gtk_text_view_get_buffer(staff->currentverse->data), FALSE);
+    //g_print("Appended <%s>\n", Lyric->str);
+  }
+  g_string_free(Lyric, FALSE);
+  Lyric = NULL;
   /* FIXME: Handle elements in other namespaces. */
 
   return 0;
@@ -2506,15 +2603,15 @@ parseScore (xmlNodePtr scoreElem, xmlNsPtr ns, DenemoGUI * gui,
 
   childElem = getXMLChild (scoreElem, "header-directives", ns);
   if (childElem != 0)
-    si->header.directives = parseDirectives(childElem, ns);
+    /*si->header.directives =*/ parseWidgetDirectives(childElem, ns, header_directive_put_graphic);
 
   childElem = getXMLChild (scoreElem, "layout-directives", ns);
   if (childElem != 0)
-    si->layout.directives = parseDirectives(childElem, ns);
+    /*si->layout.directives =*/ parseWidgetDirectives(childElem, ns, layout_directive_put_graphic);
 
   childElem = getXMLChild (scoreElem, "movementcontrol-directives", ns);
   if (childElem != 0)
-    si->movementcontrol.directives = parseDirectives(childElem, ns);
+    /*si->movementcontrol.directives =*/ parseWidgetDirectives(childElem, ns, movementcontrol_directive_put_graphic);
 
   childElem = getXMLChild (scoreElem, "score-info", ns);
   RETURN_IF_ELEM_NOT_FOUND ("score", childElem, "score-info");
@@ -2608,7 +2705,6 @@ importXML (gchar * filename, DenemoGUI *gui, ImportType type)
       ret = -1;
       return ret;
     }
-  g_signal_handlers_block_by_func(G_OBJECT (gui->scorearea), G_CALLBACK (scorearea_expose_event), NULL);
 
   /*
    * Do a couple of sanity checks to make sure we've actually got a Denemo
@@ -2718,10 +2814,10 @@ importXML (gchar * filename, DenemoGUI *gui, ImportType type)
 	      parseSetupInfo(childElem, ns, gui);
 	    } else
 	      if (ELEM_NAME_EQ (childElem, "scoreheader-directives")){
-		gui->scoreheader.directives = parseDirectives(childElem, ns);
+		/*gui->scoreheader.directives =*/ parseWidgetDirectives(childElem, ns, scoreheader_directive_put_graphic);
 	      } else
 		if (ELEM_NAME_EQ (childElem, "paper-directives")){
-		  gui->paper.directives = parseDirectives(childElem, ns);
+		  /*gui->paper.directives =*/ parseWidgetDirectives(childElem, ns, paper_directive_put_graphic);
 		} else
 		  if (ELEM_NAME_EQ (childElem, "custom_scoreblock")){
 		    gchar *tmp = (gchar *) xmlNodeListGetString (childElem->doc,
@@ -2796,7 +2892,7 @@ importXML (gchar * filename, DenemoGUI *gui, ImportType type)
   g_hash_table_destroy (sXMLIDToElemMap);
   sXMLIDToElemMap = NULL;
 
-  g_signal_handlers_unblock_by_func(G_OBJECT (gui->scorearea), G_CALLBACK (scorearea_expose_event), NULL);
+
   //g_print("Number of movements %d\n", g_list_length(gui->movements));
   return ret;
 }
