@@ -37,7 +37,7 @@
 #endif
 
 #define DEFAULT_KEYMAP "Default.cmdset"
-//index of columns in the keymap command list store FIXME if you add columns you must add them in keymap_get_command_row !!!!!!!!!
+//index of columns in the keymap command list store FIXME if you add columns you must add them in keymap_get_command_row and allocate_keymap !!!!
 enum {
     COL_TYPE = 0,
     COL_ACTION,
@@ -46,6 +46,8 @@ enum {
     COL_TOOLTIP,
     COL_CALLBACK,
     COL_BINDINGS,
+    COL_HIDDEN,
+    COL_DELETED,
     N_COLUMNS
 };
 
@@ -57,10 +59,12 @@ typedef struct command_row {
   gchar *tooltip;
   gpointer callback;
   GtkListStore *bindings;
+  gboolean hidden;
+  gboolean deleted;
 }command_row;
 
 static void
-load_keymap_file_named (keymap * the_keymap, gchar *keymapfile, gchar *fallback);
+load_keymap_file_named (gchar *keymapfile, gchar *fallback);
 
 /* Returns the state of the event after removing the modifiers consumed by the
  * system and unwanted modifiers. Use this before doing anything based on the
@@ -73,6 +77,26 @@ dnm_sanitize_key_state(GdkEventKey *event)
     GdkModifierType consumed;
     /* We want to ignore irrelevant modifiers like ScrollLock */
 
+    gdk_keymap_translate_keyboard_state (NULL, event->hardware_keycode,
+        event->state, event->group, NULL, NULL, NULL, &consumed);
+    /* removing consumed modifiers from ret */
+    ret &= ~consumed;
+    /* removing other unwanted modifiers from event->state */
+    ret &= (GDK_CONTROL_MASK | GDK_SHIFT_MASK | GDK_MOD1_MASK
+
+ | GDK_MOD2_MASK | GDK_MOD3_MASK | GDK_MOD4_MASK | GDK_MOD5_MASK    /*    these make numlock required to be off for example */);
+    return ret;
+}
+/* Returns the state of the event after removing the modifiers consumed by the
+ * system and even more unwanted modifiers. Use this if sanitize is insufficient.
+ */
+guint
+dnm_hyper_sanitize_key_state(GdkEventKey *event)
+{
+    guint ret = event->state;
+    GdkModifierType consumed;
+    /* We want to ignore irrelevant modifiers like ScrollLock */
+
     gdk_keymap_translate_keyboard_state (NULL, event->hardware_keycode,
         event->state, event->group, NULL, NULL, NULL, &consumed);
     /* removing consumed modifiers from ret */
@@ -82,6 +106,28 @@ dnm_sanitize_key_state(GdkEventKey *event)
     return ret;
 }
 
+/* Returns the state of the event as for hyper_sanitize
+ * additionally undoes the effect of CapsLock on keyval when Shift is not pressed.
+ * note that event->keycode & keystring are not altered, leaving event inconsistent.
+ * this could conceivably become a problem in other developments.
+ * Use this if hyper sanitize is insufficient.
+ */
+guint
+dnm_meta_sanitize_key_state(GdkEventKey *event)
+{
+    guint ret = event->state;
+    if(ret&GDK_LOCK_MASK) {
+      if(!(ret&GDK_SHIFT_MASK))
+	event->keyval += ('a'-'A');
+    }
+      
+    /* removing everything other than control shift and alt modifiers from event->state */
+    ret &= (GDK_CONTROL_MASK | GDK_SHIFT_MASK | GDK_MOD1_MASK);
+    return ret;
+}
+
+
+
 /*
  * Returns True if the key event is just a modifier key, False otherwise
  * TODO look for a gdk function doing that properly
@@ -92,7 +138,7 @@ isModifier(GdkEventKey *event)
     /* This check for modifier values on the event may not be right,
        if the contents of gdkkeysyms.h are OS-dependent. I don't believe
        they are. */
-    return event->keyval >= GDK_Shift_L && event->keyval <= GDK_Hyper_R;
+  return (event->keyval >= GDK_Shift_L && event->keyval <= GDK_Hyper_R) || (event->keyval==GDK_Num_Lock);
 }
 
 static inline gboolean
@@ -518,13 +564,19 @@ keymap *allocate_keymap(void)
 					    G_TYPE_POINTER,//label
 					    G_TYPE_POINTER,//tooltip
 					    G_TYPE_POINTER,//callback
-					    GTK_TYPE_LIST_STORE);
+					    GTK_TYPE_LIST_STORE,//bindings
+					    G_TYPE_BOOLEAN, //hidden
+					    G_TYPE_BOOLEAN //deleted
+					    );
   
   //empty index reference
   the_keymap->idx_from_name =
       g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
   the_keymap->idx_from_keystring =
-      g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free); 
+      g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+
+  the_keymap->cursors = g_hash_table_new(g_int_hash, g_int_equal);
+  //  g_print("Created hash table %p\n", the_keymap->cursors);
   return the_keymap;
 }
 
@@ -559,7 +611,7 @@ register_command(keymap *the_keymap, GtkAction *action, const gchar *name, const
 #if DEBUG
         //This code is only relevant to developers, to check that no action
         //entry masks another. Users cannot add actions. THIS IS CHANGING NOW...
-        gint idx = lookup_index_from_name(the_keymap, name);
+        gint idx = lookup_command_from_name(the_keymap, name);
         if (idx != -1) {
             g_warning("Command %s is inserted more than once, aborting...\n",
                     name);
@@ -656,6 +708,8 @@ keymap_get_command_row(keymap *the_keymap, command_row *row, guint command_idx)
 		       COL_CALLBACK, &row->callback,
 		       COL_TOOLTIP, &row->tooltip,
 		       COL_BINDINGS, &row->bindings,
+		       COL_HIDDEN, &row->hidden,
+		       COL_DELETED, &row->deleted,
 		       -1);
     return TRUE;
 }
@@ -748,20 +802,38 @@ keymap_foreach_command_binding (keymap *the_keymap, guint command_idx,
 }
 /**
  *  Search through keybindings for a specific binding
- *
+ *  return the command idx for the command that is bound to the keyval,state pair, or -1 if none
  */
 gint
-lookup_keybinding (keymap * the_keymap, gint keyval, GdkModifierType state)
+lookup_command_for_keybinding (keymap * the_keymap, gint keyval, GdkModifierType state)
 {
   gint res;
   gchar *name = dnm_accelerator_name(keyval, state);
-  res = lookup_keybinding_from_string(the_keymap, name);
+  res = lookup_command_for_keybinding_name(the_keymap, name);
   g_free(name);
   return res;
 }
 
+/* weaker lookup of keybinding */
+gint lookup_command_for_keyevent(GdkEventKey * event) {
+  keymap *the_keymap = Denemo.map;
+  gint command_idx = lookup_command_for_keybinding (the_keymap, event->keyval,
+						    dnm_sanitize_key_state(event));
+  if(!Denemo.prefs.strictshortcuts){
+    if(command_idx==-1)
+      command_idx = lookup_command_for_keybinding (the_keymap, event->keyval,
+						   dnm_hyper_sanitize_key_state(event)); 
+    if(command_idx==-1)
+      command_idx = lookup_command_for_keybinding (the_keymap, event->keyval,
+						   dnm_meta_sanitize_key_state(event));
+  }  
+  return command_idx;
+}
+
+
+/* looks up the command idx for the binding of name binding_name */
 gint
-lookup_keybinding_from_string (keymap * the_keymap,
+lookup_command_for_keybinding_name (keymap * the_keymap,
         const gchar *binding_name)
 {
   gpointer *value = g_hash_table_lookup (the_keymap->idx_from_keystring,
@@ -772,36 +844,7 @@ lookup_keybinding_from_string (keymap * the_keymap,
     return -1;
 }
 
-/**
- * Look up a key binding by name.
- * FIXME: This is inefficent. The keybinding structures needs a rework to be more usefull and robust.
- *
- * deprecated and not reimplemented, developper should use
- * keymap_foreach_command_binding
- * 
- * @param keymap
- * @param name
- * @returns the list of keybinding, or NULL if not found
- */
-/*
-GList *
-lookup_keybindings_by_name (keymap * keymap, const gchar * name)
-{
-  gpointer value = g_hash_table_lookup(keymap->idx_from_name, name);
-  if (value)
-      return lookup_keybindings_by_idx(keymap, *(guint *)value);
-  else {
-      g_warning ("Could not find command '%s'\n", name);
-      return NULL;
-  }
-}
 
-GList *
-lookup_keybindings_by_idx (keymap * keymap, guint idx)
-{
-  return g_array_index(keymap->bindings, GList *, idx);
-}
-*/
 
 /**
  * Look up for a command index.
@@ -810,7 +853,7 @@ lookup_keybindings_by_idx (keymap * keymap, guint idx)
  * @param name
  */
 gint 
-lookup_index_from_name (keymap * keymap, const gchar *command_name)
+lookup_command_from_name (keymap * keymap, const gchar *command_name)
 {
   gpointer value = g_hash_table_lookup(keymap->idx_from_name, command_name);
   if (value)
@@ -872,6 +915,28 @@ lookup_tooltip_from_idx (keymap * keymap, guint command_idx)
   return res;
 }
 
+gboolean lookup_hidden_from_idx (keymap * keymap, guint command_idx)
+{
+  gboolean res = FALSE;
+  command_row row;
+  if (!keymap_get_command_row(keymap, &row, command_idx))
+      return NULL;
+  res = row.hidden;//FIXME label is a property g_object_get_prop...
+  g_object_unref(row.bindings);
+  return res;
+}
+
+
+gboolean lookup_deleted_from_idx (keymap * keymap, guint command_idx)
+{
+  gboolean res = FALSE;
+  command_row row;
+  if (!keymap_get_command_row(keymap, &row, command_idx))
+      return NULL;
+  res = row.deleted;//FIXME label is a property g_object_get_prop...
+  g_object_unref(row.bindings);
+  return res;
+}
 
 
 //do not free the result
@@ -1010,12 +1075,12 @@ void
 remove_keybinding (keymap * the_keymap, gint keyval, GdkModifierType state)
 {
   gchar *name = dnm_accelerator_name(keyval, state);
-  remove_keybinding_from_string(the_keymap, name);
+  remove_keybinding_from_name(the_keymap, name);
   g_free(name);
 }
 
 void
-remove_keybinding_from_string (keymap * the_keymap, const gchar *binding)
+remove_keybinding_from_name (keymap * the_keymap, const gchar *binding)
 {
   gint *value;
   value = (gint *) g_hash_table_lookup(the_keymap->idx_from_keystring, binding);
@@ -1028,10 +1093,11 @@ remove_keybinding_from_string (keymap * the_keymap, const gchar *binding)
 
 /*
  * Insert a binding to the bindings of command_idx.
+ * pos indicates where in the list of bindings to insert this binding.
  */
 static void
 add_keybinding_bindings_helper(keymap *the_keymap, guint command_idx,
-        const gchar *binding, KbdPosition pos)
+        const gchar *binding, ListPosition pos)
 {
   command_row row;
   GtkTreeIter iter;
@@ -1055,8 +1121,8 @@ add_keybinding_bindings_helper(keymap *the_keymap, guint command_idx,
 }
 
 gint
-add_keybinding_from_name(keymap * the_keymap, gint keyval,
-        GdkModifierType state, const gchar *command_name, KbdPosition pos) 
+add_keybinding_to_named_command(keymap * the_keymap, gint keyval,
+        GdkModifierType state, const gchar *command_name, ListPosition pos) 
 {
   gpointer value;
   guint command_idx;
@@ -1066,49 +1132,71 @@ add_keybinding_from_name(keymap * the_keymap, gint keyval,
       return -1;
   }
   command_idx = *(guint *) value;
-  return add_keybinding_from_idx(the_keymap, keyval, state, command_idx, pos);
+  return add_keybinding_to_idx(the_keymap, keyval, state, command_idx, pos);
 }
+ 
+gint
+add_named_binding_to_idx (keymap * the_keymap, gchar *kb_name,  guint command_idx, ListPosition pos)
+{
+  guint *new_idx;
+  gint old_command_idx;
+  old_command_idx = lookup_command_for_keybinding_name(the_keymap, kb_name);//lookup_keybinding(the_keymap, keyval, state);  
+  gchar *title = NULL;
+  gchar *prompt = NULL;
+  if(old_command_idx >= 0) {
+    title = g_strdup_printf("The Command %s Responds to this Shortcut", lookup_name_from_idx(Denemo.map, old_command_idx));
+    prompt = g_strdup_printf("Lose the shortcut %s for this?", kb_name);
+  }
+  
+  if((pos==POS_FIRST) && 
+     (old_command_idx >= 0) &&
+     (!confirm(title, prompt))){
+      g_free(title);
+      g_free(prompt);
+      return  old_command_idx;
+    }
+  
+  if(old_command_idx >= 0) {
+    remove_keybinding_bindings_helper(the_keymap, old_command_idx, kb_name);
+    update_accel_labels(the_keymap, old_command_idx);
+  } 
+  //add the keybinding to the binding on idx_command
+  add_keybinding_bindings_helper(the_keymap, command_idx, kb_name, pos);
     
+  //update the accel labels of the command
+  update_accel_labels(the_keymap, command_idx);
+    
+  //add or modify an entry in idx_from_keystring
+  new_idx = (guint *) g_malloc(sizeof(guint));
+  *new_idx = command_idx;
+  g_hash_table_insert(the_keymap->idx_from_keystring, g_strdup(kb_name),
+			new_idx);  
+  g_free(title);
+  g_free(prompt);
+  return old_command_idx;
+}
+  
+   
 /**
  * Adds a keybinding to the_keymap.  If the key was already bound,
  * this function removes the old binding and replaces it, returning
  * the number of the command this keybinding was attached to. Otherwise
  * returns -1. 
+ * if pos is POS_FIRST, then the user is adding the binding - get confirmation before stealing
+ * if POS_LAST it is the command set being loaded, do not ask
  */
 gint
-add_keybinding_from_idx (keymap * the_keymap, gint keyval,
-        GdkModifierType state, guint command_idx, KbdPosition pos)
+add_keybinding_to_idx (keymap * the_keymap, gint keyval,
+        GdkModifierType state, guint command_idx, ListPosition pos)
 {
-  guint *new_idx;
   gint old_command_idx;
   gpointer value;
   gchar *kb_name;
   gboolean flag_update_accel;
-  
   kb_name = dnm_accelerator_name(keyval, state);
-  old_command_idx = lookup_keybinding(the_keymap, keyval, state);
-  
-  //if the keybinding was previously used, remove it from bindings and update
-  //its accels
-  if (old_command_idx != -1) {
-      remove_keybinding_bindings_helper(the_keymap, old_command_idx, kb_name);
-      update_accel_labels(the_keymap, old_command_idx);
-  }  
-
-  //add the keybinding to the binding on idx_command
-   add_keybinding_bindings_helper(the_keymap, command_idx, kb_name, pos);
-
-  //update the accel labels of the command
-  update_accel_labels(the_keymap, command_idx);
-
-  //add or modify an entry in idx_from_keystring
-  new_idx = (guint *) g_malloc(sizeof(guint));
-  *new_idx = command_idx;
-  g_hash_table_insert(the_keymap->idx_from_keystring, g_strdup(kb_name),
-          new_idx);
-  
+  old_command_idx = add_named_binding_to_idx (the_keymap, kb_name, command_idx, pos);
   g_free(kb_name);
-  Denemo.accelerator_status = TRUE;
+  //Denemo.accelerator_status = TRUE;
   return old_command_idx;
 }
 
@@ -1129,13 +1217,12 @@ stolen_gtk_menu_stop_navigating_submenu (GtkMenu *menu)
 
 
 gint
-keymap_accel_quick_edit_snooper(GtkWidget *grab_widget, GdkEventKey *event,
-		gpointer func_data)
+keymap_accel_quick_edit_snooper(GtkWidget *grab_widget, GdkEventKey *event)
 {
   guint keyval;
   GdkModifierType modifiers;
   GtkAction *action;
-  keymap *the_keymap = Denemo.commands;
+  keymap *the_keymap = Denemo.map;
   GtkMenu *menu = GTK_MENU(grab_widget);
   GtkMenuClass *menu_class = GTK_MENU_GET_CLASS(menu);
   GtkMenuShellClass *parent_class = g_type_class_peek_parent(menu_class);
@@ -1169,13 +1256,13 @@ keymap_accel_quick_edit_snooper(GtkWidget *grab_widget, GdkEventKey *event,
  //If this menu item has no action we give up
   if (!action)
     return TRUE;
-  gint idx = lookup_index_from_name(the_keymap, gtk_action_get_name(action));
+  gint idx = lookup_command_from_name(the_keymap, gtk_action_get_name(action));
   //If this menu item  action is not registered in the
   //keymap, we give up
   if (idx == -1)
     return TRUE;
   //Add the keybinding
-  add_keybinding_from_idx(the_keymap, keyval, modifiers, idx, POS_FIRST);
+  add_keybinding_to_idx(the_keymap, keyval, modifiers, idx, POS_FIRST);
   return TRUE;
 }
 gboolean
@@ -1194,7 +1281,7 @@ idx_has_callback(keymap *the_keymap, guint command_idx)
 
 
 
-GtkAction *action_of_name(keymap *the_keymap, gchar *command_name) { 
+GtkAction *lookup_action_from_name (gchar *command_name) { 
   // GtkActionGroup *action_group = get_action_group(the_keymap);
   return gtk_action_group_get_action(Denemo.action_group, command_name);
 }
@@ -1240,7 +1327,7 @@ dump_command_info (keymap *the_keymap, gint command_idx)
   do {
     gtk_tree_model_get(model_bind, &iter, 0, &cur_binding, -1);
     g_print("\t%s (%d)\n", cur_binding,
-            lookup_keybinding_from_string(the_keymap, cur_binding));
+            lookup_command_for_keybinding_name(the_keymap, cur_binding));
     g_free(cur_binding);
   } while (gtk_tree_model_iter_next(model_bind, &iter));
   g_object_unref(row.bindings);
@@ -1283,8 +1370,9 @@ locatekeymapdir ()
 
 
 /**
+ *  loads a command set (aka keymap) from a file in a file selector.
  * This function is a callback that is wrapper for
- * load_keymap_file 
+ * load_keymap_file amongst others
  *FIXME note that non xml file support has been commented out
  */
 void
@@ -1293,18 +1381,20 @@ load_keymap_from_dialog (GtkWidget * widget, GtkWidget *filesel)
   gchar *name = (gchar *)
     gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
   if(g_file_test (name, G_FILE_TEST_EXISTS))
-     load_keymap_file_named(Denemo.commands, NULL, name);
+     load_keymap_file_named(NULL, name);
+  Denemo.accelerator_status = TRUE;
 }
 
-/**
- * Function for loading a keymap from location by way of
- * a user dialog. 
- */
+
 static 	void show_type(GtkWidget *widget, gchar *message) {
     g_print("%s%s\n",message, widget?g_type_name(G_TYPE_FROM_INSTANCE(widget)):"NULL widget");
   }
+/**
+ * Function for loading a command set (aka keymap) from location by way of
+ * a user dialog. 
+ */
 void
-load_keymap_dialog_location (GtkWidget * widget, keymap * the_keymap, gchar *location)
+load_keymap_dialog_location (GtkWidget * widget, gchar *location)
 {
   GtkWidget *filesel;
   filesel = gtk_file_selection_new (_("Load Command Set"));
@@ -1337,7 +1427,7 @@ load_keymap_dialog (GtkWidget * widget)
 {
   gchar *keymapdir = g_strdup_printf("%s%c", locatekeymapdir(),G_DIR_SEPARATOR);
   if(keymapdir)
-    load_keymap_dialog_location (widget, Denemo.commands, keymapdir);
+    load_keymap_dialog_location (widget, keymapdir);
   else
     warningdialog("Cannot access your local .denemo");
   g_free(keymapdir);
@@ -1349,7 +1439,7 @@ load_system_keymap_dialog (GtkWidget * widget)
   gchar *systemwide = g_build_filename (get_data_dir (), "actions", DEFAULT_KEYMAP,
                                         NULL);
   if(systemwide)
-    load_keymap_dialog_location (widget, Denemo.commands, systemwide);
+    load_keymap_dialog_location (widget, systemwide);
   else
     warningdialog("Installation error");
   g_free(systemwide);
@@ -1363,16 +1453,14 @@ load_system_keymap_dialog (GtkWidget * widget)
 
  */
 static void
-load_keymap_file_named (keymap * the_keymap, gchar *localrc, gchar *systemwide) {
-
-  // keymap_clear_bindings (the_keymap); this doesn't prevent keybindings hanging around???
+load_keymap_file_named (gchar *localrc, gchar *systemwide) {
 
   if(localrc) {
     g_print ("Trying local file %s as xml...", localrc);
-    if (load_xml_keymap (localrc, the_keymap) == -1)
+    if (load_xml_keymap (localrc) == -1)
       {
 	g_print ("..no.\nTrying systemwide file %s as xml...", systemwide);
-	if (load_xml_keymap (systemwide, the_keymap) == -1)
+	if (load_xml_keymap (systemwide) == -1)
 	  {
 	    g_print ("..no.\nNo useful keymaps found.\n");
 	    no_map_dialog ();
@@ -1384,8 +1472,8 @@ load_keymap_file_named (keymap * the_keymap, gchar *localrc, gchar *systemwide)
       g_print ("..ok.\n");
   }
   else {
-    if (load_xml_keymap (systemwide, the_keymap) == -1)
-      warningdialog("Could not load keymap file selected");
+    if (load_xml_keymap (systemwide) == -1)
+      warningdialog("Could not load command set file");
   }
 }
 
@@ -1394,7 +1482,7 @@ load_keymap_file_named (keymap * the_keymap, gchar *localrc, gchar *systemwide)
  * or (if that doesn't load) the global default keymap
  */
 void
-load_default_keymap_file (keymap * the_keymap)
+load_default_keymap_file (void)
 {
   gchar *localrc = NULL;
   const gchar *keymapdir = locatekeymapdir ();
@@ -1403,7 +1491,7 @@ load_default_keymap_file (keymap * the_keymap)
   //g_print ("systemwide = %s\n", systemwide);
   if(keymapdir)
     localrc = g_build_filename (keymapdir, DEFAULT_KEYMAP, NULL);
-  load_keymap_file_named (the_keymap, localrc, systemwide);
+  load_keymap_file_named (localrc, systemwide);
   g_free(localrc);
   g_free(systemwide);
 }
@@ -1449,8 +1537,7 @@ save_keymap_from_dialog (GtkWidget * widget, GtkWidget * filesel)
 {
   save_xml_keymap ((gchar *)
 		   gtk_file_selection_get_filename (GTK_FILE_SELECTION
-						    (filesel)),
-		   Denemo.commands);
+						    (filesel)));
 
 }
 
@@ -1506,7 +1593,7 @@ save_default_keymap_file (GtkWidget *widget)
   const gchar *keymapdir = locatekeymapdir ();
   if(keymapdir)
     localrc = g_build_filename (keymapdir, DEFAULT_KEYMAP, NULL);
-  save_xml_keymap (localrc, Denemo.commands);
+  save_xml_keymap (localrc);
   g_free(localrc);
 }
 
@@ -1561,6 +1648,42 @@ command_name_data_function (GtkTreeViewColumn *col,
     g_object_set(renderer, "text", name, NULL);
 }
 
+static void
+command_hidden_data_function (GtkTreeViewColumn *col,
+                            GtkCellRenderer   *renderer,
+                            GtkTreeModel      *model,
+                            GtkTreeIter       *iter,
+                            gpointer           user_data)
+{
+    KeymapCommandType type;
+    gpointer action;
+    gboolean hidden;
+    gtk_tree_model_get(model, iter,
+            COL_TYPE, &type,
+            COL_ACTION, &action,
+            -1);
+    hidden = g_object_get_data(G_OBJECT(action), "hidden");   
+    g_object_set(renderer, "active", hidden, NULL);
+}
+
+static void
+command_deleted_data_function (GtkTreeViewColumn *col,
+                            GtkCellRenderer   *renderer,
+                            GtkTreeModel      *model,
+                            GtkTreeIter       *iter,
+                            gpointer           user_data)
+{
+    KeymapCommandType type;
+    gpointer action;
+    gboolean deleted;
+    gtk_tree_model_get(model, iter,
+            COL_TYPE, &type,
+            COL_ACTION, &action,
+            -1);
+    deleted = g_object_get_data(G_OBJECT(action), "deleted");   
+    g_object_set(renderer, "active", deleted, NULL);
+}
+
 static gboolean
 search_equal_func(GtkTreeModel *model, gint column, const gchar *key,
         GtkTreeIter *iter, gpointer search_data)
@@ -1582,6 +1705,29 @@ search_equal_func(GtkTreeModel *model, gint column, const gchar *key,
   return !res;
 }
 
+/*toggle hidden on action at row in command list */
+static void toggle_hidden_on_action (GtkCellRendererToggle *cell_renderer,
+                                            gchar *path)  {
+  gint command_idx = atoi(path);
+  GtkAction *action = lookup_action_from_idx (Denemo.map, command_idx);
+  if(GTK_IS_ACTION(action)){
+    gboolean hidden = (gboolean)g_object_get_data(action, "hidden");
+    set_visibility_for_action(action, hidden);
+  }
+}
+
+/*toggle deleted on action at row in command list */
+static void toggle_deleted_on_action (GtkCellRendererToggle *cell_renderer,
+                                            gchar *path)  {
+  gint command_idx = atoi(path);
+  GtkAction *action = lookup_action_from_idx (Denemo.map, command_idx);
+  if(GTK_IS_ACTION(action)){
+    gboolean deleted = (gboolean)g_object_get_data(action, "deleted");
+    //set_visibility_for_action(action, deleted);
+    g_object_set_data(action, "deleted", !deleted);
+  }
+}
+
 GtkWidget *
 keymap_get_command_view(keymap *the_keymap)
 {
@@ -1606,10 +1752,35 @@ keymap_get_command_view(keymap *the_keymap)
   gtk_tree_view_column_pack_start(col, renderer, TRUE);
   gtk_tree_view_column_set_cell_data_func(col, renderer,
           command_name_data_function, NULL, NULL);
- 
+
+  col = gtk_tree_view_column_new();
+  gtk_tree_view_column_set_title(col, _("Hidden"));
+  gtk_tree_view_append_column(res, col);
+
+  renderer = gtk_cell_renderer_toggle_new();
+  gtk_tree_view_column_pack_start(col, renderer, TRUE);
+  gtk_tree_view_column_add_attribute(col, renderer, "active", COL_HIDDEN);
+  gtk_tree_view_column_set_cell_data_func(col, renderer,
+					  command_hidden_data_function, NULL, NULL);
+  g_signal_connect(renderer, "toggled", toggle_hidden_on_action, NULL);
+
+
+  col = gtk_tree_view_column_new();
+  gtk_tree_view_column_set_title(col, _("Deleted"));
+  gtk_tree_view_append_column(res, col);
+
+  renderer = gtk_cell_renderer_toggle_new();
+  gtk_tree_view_column_pack_start(col, renderer, TRUE);
+  gtk_tree_view_column_add_attribute(col, renderer, "active", COL_DELETED);
+  gtk_tree_view_column_set_cell_data_func(col, renderer,
+					  command_deleted_data_function, NULL, NULL);
+  g_signal_connect(renderer, "toggled", toggle_deleted_on_action, NULL);
+
+
+
+
   selection = gtk_tree_view_get_selection(res);
   gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
-
   gtk_tree_view_set_search_equal_func(res, search_equal_func, NULL, NULL);
   gtk_tree_view_set_enable_search(res, TRUE);
   
@@ -1630,7 +1801,7 @@ void row_inserted_handler(GtkTreeModel *model, GtkTreePath *arg1,
     keyboard_dialog_data *cbdata = (keyboard_dialog_data *) user_data;
     //g_print("insert\n");
     if (cbdata->command_idx != -1)
-        update_accel_labels(Denemo.commands, cbdata->command_idx);
+        update_accel_labels(Denemo.map, cbdata->command_idx);
 }
 
 void row_deleted_handler(GtkTreeModel *model, GtkTreePath *arg1,
@@ -1639,7 +1810,7 @@ void row_deleted_handler(GtkTreeModel *model, GtkTreePath *arg1,
     keyboard_dialog_data *cbdata = (keyboard_dialog_data *) user_data;
     //g_print("delete\n");
     if (cbdata->command_idx != -1)
-        update_accel_labels(Denemo.commands, cbdata->command_idx);
+        update_accel_labels(Denemo.map, cbdata->command_idx);
 }
 
 //Performs cleanup on the keymap when a command view is closed
@@ -1735,7 +1906,7 @@ keymap_get_binding_view()
   res = GTK_TREE_VIEW(gtk_tree_view_new());
   
   col = gtk_tree_view_column_new();
-  gtk_tree_view_column_set_title(col, _("Bindings"));
+  gtk_tree_view_column_set_title(col, _("Shortcuts"));
   gtk_tree_view_column_set_sizing(col, GTK_TREE_VIEW_COLUMN_GROW_ONLY);
   gtk_tree_view_append_column(res, col);
 
