@@ -18,8 +18,9 @@
 #include "objops.h"
 
 
-int scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event);
-gboolean intercept_scorearea_keypress (gint *keyval, gint *state);
+gint scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event);
+gint scorearea_keyrelease_event (GtkWidget * widget, GdkEventKey * event);
+gboolean intercept_scorearea_keypress (GdkEventKey *pevent);
 
 void adjust_measure_less_width_key (DenemoGUI *gui);
 void adjust_measure_more_width_key (DenemoGUI *gui);
