@@ -20,6 +20,16 @@
 #include "utils.h"
 #include "prefops.h"
 
+static void edit_directive_callback(GtkWidget *w, gpointer what);
+
+
+static void
+gtk_menu_item_set_label_text(GtkMenuItem *item, gchar *text) {
+  GtkWidget *label = gtk_bin_get_child(item);
+  if(label)
+    gtk_label_set_text(label, text);
+}
+
 
 /**
  * If the curObj is a chord with a note(s)
@@ -105,6 +115,7 @@ new_directive(gchar *tag){
   return directive;
 }
 
+
 static  DenemoObject *findobj(void) {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore * si = gui->si;
@@ -716,12 +727,11 @@ DenemoObject *curObj = get_chordobject();
   return delete_directive(&thechord->directives, tag);
 }
 
-gboolean delete_score_directive(gchar *tag) {
-
-  DenemoDirective *directive = get_score_directive(tag);
+gboolean delete_score_directive(gchar *tagname) {
+  DenemoDirective *directive = get_score_directive(tagname);
   if(directive==NULL)
     return FALSE;
-  return delete_directive(&Denemo.gui->lilycontrol.directives, tag);
+  return delete_directive(&Denemo.gui->lilycontrol.directives, tagname);
 }
 
 #define GET_STR_FIELD_FUNC(what, field)\
@@ -760,6 +770,21 @@ what##_directive_put_##field(gchar *tag, gchar *value) {\
 #define PUT_STR_FIELD_FUNCV(what, field) PUT_STR_FIELD_FUNC_NAME(what, field, voice_directives)
 
 
+GET_STR_FIELD_FUNC(score, midibytes)
+GET_STR_FIELD_FUNC(movementcontrol, midibytes)
+GET_STR_FIELD_FUNC(note, midibytes)
+GET_STR_FIELD_FUNC(chord, midibytes)
+GET_STR_FIELD_FUNC(staff, midibytes)
+GET_STR_FIELD_FUNC(voice, midibytes)
+GET_STR_FIELD_FUNC(standalone, midibytes)
+
+PUT_STR_FIELD_FUNC(score, midibytes)
+PUT_STR_FIELD_FUNC(movementcontrol, midibytes)
+PUT_STR_FIELD_FUNC(note, midibytes)
+PUT_STR_FIELD_FUNC(chord, midibytes)
+PUT_STR_FIELD_FUNCS(staff, midibytes)
+PUT_STR_FIELD_FUNCV(voice, midibytes)
+
 
 GET_STR_FIELD_FUNC(chord, prefix)
 GET_STR_FIELD_FUNC(chord, postfix)
@@ -780,7 +805,7 @@ PUT_STR_FIELD_FUNC(note, display)
 
 PUT_STR_FIELD_FUNC(score, prefix)
 PUT_STR_FIELD_FUNC(score, postfix)
-PUT_STR_FIELD_FUNC(score, display)
+
 
 GET_STR_FIELD_FUNC(standalone, prefix)
 GET_STR_FIELD_FUNC(standalone, postfix)
@@ -796,7 +821,7 @@ GET_STR_FIELD_FUNC(staff, display)
 
 PUT_STR_FIELD_FUNCS(staff, prefix)
 PUT_STR_FIELD_FUNCS(staff, postfix)
-PUT_STR_FIELD_FUNCS(staff, display)
+
 
 GET_STR_FIELD_FUNC(voice, prefix)
 GET_STR_FIELD_FUNC(voice, postfix)
@@ -804,7 +829,7 @@ GET_STR_FIELD_FUNC(voice, display)
 
 PUT_STR_FIELD_FUNCV(voice, prefix)
 PUT_STR_FIELD_FUNCV(voice, postfix)
-PUT_STR_FIELD_FUNCV(voice, display)
+
 #undef staff
 
 #define PUT_INT_FIELD_FUNC_NAME(what, field, name)\
@@ -886,7 +911,6 @@ PUT_INT_FIELD_FUNC(chord, override)
 PUT_INT_FIELD_FUNCS(staff, override)
 PUT_INT_FIELD_FUNCV(voice, override)
 PUT_INT_FIELD_FUNC(score, override)
-     //standalone has nothing to override
 GET_INT_FIELD_FUNC(note, override)
 GET_INT_FIELD_FUNC(chord, override)
 GET_INT_FIELD_FUNC(staff, override)
@@ -955,7 +979,7 @@ GET_INT_FIELD_FUNC(chord, gy)
 GET_INT_FIELD_FUNC(staff, gy)
 GET_INT_FIELD_FUNC(voice, gy)
 
-
+GET_INT_FIELD_FUNC(standalone, override)
 GET_INT_FIELD_FUNC(standalone, x)
 GET_INT_FIELD_FUNC(standalone, y)
 
@@ -1024,6 +1048,95 @@ what##_directive_put_graphic(gchar *tag, gchar *value) {\
   return TRUE;\
 }
 
+/*
+  label_widget()
+  if directive does not have graphic:
+     fn gives the type of directive, which must be a non-DenemoObject directive: it determines where the widget goes (score or movement level, DenemoGUI or DenemoScore respectively).
+     field must be "display" or "graphic_name"
+     creates a widget (button or menu depending on fn) for editing directive, point directive->graphic to it and attach a callback to edit this directive, passing fn as data to it (to say what sort of directive it is).
+     places the widget in the appropriate buttonbox.
+  if directive does have graphic:
+     fn and field are ignored.
+  In all cases:
+     set  the label for the widget from directive->graphic_name directive->display
+
+*/
+     
+static void 
+label_widget(DenemoDirective *directive,  void fn(), gchar *field) {
+  GtkWidget *box;
+  gchar *value = g_strdup_printf("%s%s",directive->graphic_name?directive->graphic_name->str:"" , 
+				 directive->display?directive->display->str:"");
+
+  //  g_print("Have value %s\n", value);
+  //if this is not the display field, but the graphic that is being set, create the graphic
+  if((directive->graphic==NULL) && *value && strcmp(field, "display") ) {
+    if(fn==score_directive_put_graphic ||fn==scoreheader_directive_put_graphic ||fn==paper_directive_put_graphic)  
+      box = Denemo.gui->buttonbox;
+    else
+      box = Denemo.gui->si->buttonbox;
+    if(fn==staff_directive_put_graphic) {
+      //g_print("Doing the staff case");
+      directive->graphic = gtk_menu_item_new_with_label(value);
+      /* g_print("directive-type %s.....", thetype);	*/
+      GtkWidget *menu;
+      menu = ((DenemoStaff*)Denemo.gui->si->currentstaff->data)->staffmenu;
+      g_signal_connect(G_OBJECT(directive->graphic), "activate",  G_CALLBACK(edit_directive_callback), fn);
+      gtk_menu_shell_append(GTK_MENU_SHELL(menu), directive->graphic);
+    } else    
+      if(fn==voice_directive_put_graphic) {
+	//g_print("Doing the voice case");
+	directive->graphic = gtk_menu_item_new_with_label(value);//WARNING _with_label is important
+	/* g_print("directive-type %s.....", thetype);	*/
+	GtkWidget *menu;
+	menu = ((DenemoStaff*)Denemo.gui->si->currentstaff->data)->voicemenu;  
+	g_signal_connect(G_OBJECT(directive->graphic), "activate",  G_CALLBACK(edit_directive_callback), fn);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), directive->graphic);
+      }  else {
+	//g_print("Doing the non-staff case");
+	directive->graphic = gtk_button_new_with_label(value);
+	g_signal_connect(G_OBJECT(directive->graphic), "clicked",  G_CALLBACK(edit_directive_callback), fn);
+	gtk_box_pack_start (GTK_BOX (box), directive->graphic, FALSE, TRUE,0);
+	gtk_widget_show(directive->graphic);
+	gtk_widget_show(box);
+      }
+    g_object_set_data(directive->graphic, "directive", (gpointer)directive);
+    gtk_widget_show_all(GTK_WIDGET(directive->graphic));
+    GTK_WIDGET_UNSET_FLAGS(directive->graphic, GTK_CAN_FOCUS);
+  }//not display field
+  if(directive->graphic){
+    if(GTK_IS_BUTTON(directive->graphic))
+      gtk_button_set_label(directive->graphic, value);
+    else
+      gtk_menu_item_set_label_text(directive->graphic, value);
+  }
+  g_free(value);
+}
+
+
+// create a directive for non-DenemoObject directive #what
+// create a button to edit the directive as the directive->graphic
+#define PUT_GRAPHIC_WIDGET_FUNC(field, field_name, what, directives) gpointer \
+what##_directive_put_##field(gchar *tag, gchar *value) {\
+  what *current = get_##what();\
+  if(current==NULL) return NULL;\
+  if(current->directives==NULL)\
+       create_directives (&current->directives, tag);\
+  DenemoDirective *directive = get_##what##_directive(tag);\
+  if(directive==NULL){\
+    directive=new_directive(tag);\
+    current->directives = g_list_append(current->directives, directive);\
+    }\
+  if(directive->field_name==NULL) \
+    directive->field_name = g_string_new(value);\
+  else\
+    g_string_assign(directive->field_name, value);\
+  label_widget(directive, what##_directive_put_graphic, #field);\
+  return (gpointer)directive;\
+}
+
+
+
 #define PUT_GRAPHIC(what) PUT_GRAPHIC_NAME(what, directives)
 #define PUT_GRAPHICS(what) PUT_GRAPHIC_NAME(what, staff_directives)
 #define PUT_GRAPHICV(what) PUT_GRAPHIC_NAME(what, voice_directives)
@@ -1031,9 +1144,31 @@ what##_directive_put_graphic(gchar *tag, gchar *value) {\
 
 PUT_GRAPHIC(chord);
 PUT_GRAPHIC(note);
-PUT_GRAPHIC(score);
-PUT_GRAPHICS(staff);
-PUT_GRAPHICV(voice);
+
+#define PUT_GRAPHIC_WIDGET(what, directives) PUT_GRAPHIC_WIDGET_FUNC(graphic, graphic_name, what, directives)
+#define PUT_DISPLAY_FUNC(what, directives) PUT_GRAPHIC_WIDGET_FUNC(display, display, what, directives)
+PUT_GRAPHIC_WIDGET(score, directives);
+PUT_GRAPHIC_WIDGET(staff, staff_directives);
+PUT_GRAPHIC_WIDGET(voice, voice_directives);
+PUT_GRAPHIC_WIDGET(scoreheader, directives);
+PUT_GRAPHIC_WIDGET(header, directives);
+PUT_GRAPHIC_WIDGET(paper, directives);
+PUT_GRAPHIC_WIDGET(layout, directives);
+PUT_GRAPHIC_WIDGET(movementcontrol, directives);
+
+PUT_DISPLAY_FUNC(score, directives);
+PUT_DISPLAY_FUNC(staff, staff_directives);
+PUT_DISPLAY_FUNC(voice, voice_directives);
+PUT_DISPLAY_FUNC(scoreheader, directives);
+PUT_DISPLAY_FUNC(header, directives);
+PUT_DISPLAY_FUNC(paper, directives);
+PUT_DISPLAY_FUNC(layout, directives);
+PUT_DISPLAY_FUNC(movementcontrol, directives);
+
+#undef PUT_GRAPHIC_WIDGET
+#undef PUT_DISPLAY_FUNC
+#undef PUT_GRAPHIC_WIDGET_FUNC
+
 
 
 gboolean
@@ -1083,6 +1218,7 @@ standalone_directive_put_##field(gchar *tag, gchar *value) {\
 STANDALONE_PUT_STR_FIELD_FUNC(prefix);
 STANDALONE_PUT_STR_FIELD_FUNC(postfix);
 STANDALONE_PUT_STR_FIELD_FUNC(display);
+STANDALONE_PUT_STR_FIELD_FUNC(midibytes);
 
 
 
@@ -1113,6 +1249,8 @@ STANDALONE_PUT_INT_FIELD_FUNC(ty);
 STANDALONE_PUT_INT_FIELD_FUNC(gx);
 STANDALONE_PUT_INT_FIELD_FUNC(gy);
 
+STANDALONE_PUT_INT_FIELD_FUNC(override);
+
 
 gboolean 
 standalone_directive_put_minpixels(gchar *tag, gint value) {
@@ -1132,31 +1270,7 @@ standalone_directive_put_minpixels(gchar *tag, gint value) {
   return TRUE;
 }
 
-/* returns the path to a script file for editing a directive created by command of name commandname 
-   a local one takes precedence over the system one
-   caller must g_free the result */
-static gchar *
-get_script_file(gchar *commandname) {
-  gchar *name = g_strconcat(commandname, ".scm", NULL);
-  gchar* filename = g_build_filename (locatedotdenemo(), "actions", "editscripts", name, NULL);
-  if(!g_file_test(filename, G_FILE_TEST_EXISTS)) {
-    g_free(filename);
-    filename = g_build_filename(get_data_dir(), "actions", "editscripts", name, NULL);
-    if(g_file_test(filename, G_FILE_TEST_EXISTS))
-      return filename;
-    g_free(filename);
-    return NULL;
-  }
-  return filename;
-}
 
-static gboolean
-script_file_exists(gchar *commandname){
-  commandname = get_script_file(commandname);
-  if(commandname==NULL) return FALSE;
-  g_free(commandname);
-  return TRUE;
-}
 
 static gboolean
 tag_choice(GtkWidget *widget, DenemoDirective **response) {
@@ -1233,36 +1347,44 @@ DenemoDirective *select_directive(gchar *instr, GList *directives) {
 
 /* let the user choose from the directives at the cursor */
 static 
-void user_select_directive_at_cursor(GList ***pdirectives, DenemoDirective **pdirective) {
+void user_select_directive_at_cursor(gchar **what, GList ***pdirectives, DenemoDirective **pdirective) {
   *pdirectives = NULL;
   *pdirective = get_standalone_directive(NULL);
   if(*pdirective)
     return;
+  gchar *name=NULL;
   note *curnote = get_note();
-  if(curnote==NULL)
-    return;//if we allow chord directives on rests this must change.
-  gchar *name = mid_c_offsettolily(curnote->mid_c_offset, curnote->enshift);
-  if(curnote->mid_c_offset == Denemo.gui->si->cursor_y)
-    if(curnote->directives) {
-      *pdirectives = &curnote->directives;
-      gchar *instr = g_strdup_printf("Select a directive attached to the note \"%s\"", name);
-      *pdirective = select_directive(instr, **pdirectives);
-      g_free(instr);
-      if(*pdirective)
-	return;
-    }
+  if(curnote!=NULL) {
+    name = mid_c_offsettolily(curnote->mid_c_offset, curnote->enshift);
+    if(curnote->mid_c_offset == Denemo.gui->si->cursor_y)
+      if(curnote->directives) {
+	*pdirectives = &curnote->directives;
+	*what = "note";
+	gchar *instr = g_strdup_printf("Select a directive attached to the note \"%s\"", name);
+	*pdirective = select_directive(instr, **pdirectives);
+	g_free(instr);
+	if(*pdirective) {
+	  g_free(name);
+	  return;
+	}
+      }
+  }
+  
+
   {
   // not exactly on a note, offer any chord directives
     gchar *instr = "Select a directive attached to the chord";
     chord *curchord = get_chord();
     if(curchord && curchord->directives) {
       *pdirectives = &curchord->directives;
+      *what = "chord";
       *pdirective = select_directive(instr, **pdirectives);
     } 
   }
-  if(*pdirective==NULL)//try nearest note
+  if(*pdirective==NULL && curnote)//try nearest note
     if(curnote->directives && curnote->mid_c_offset != Denemo.gui->si->cursor_y) {
       *pdirectives = &curnote->directives;
+      *what = "note";
       gchar *instr = g_strdup_printf("Select a directive attached to the note \"%s\"", name);
       *pdirective = select_directive(instr, **pdirectives);
       g_free(instr);
@@ -1338,12 +1460,66 @@ static gboolean set_int (GtkSpinButton *widget,  gint *val)  {
   return TRUE;
 }
 
+static gchar* quote_scheme(gchar *s) {
+  GString *dest = g_string_new("");
+  gchar *c;
+  for(c=s;*c;c++) {
+    if(*c=='"' || *c=='\\')
+      g_string_append_c(dest, '\\');
+    g_string_append_c(dest, *c);  
+  }
+  return g_string_free(dest, FALSE);
+}
+
+static void create_script(DenemoDirective *directive, gchar *what) {
+  GString *scheme = g_string_new(directive->tag->str);
+  g_string_prepend(scheme, ";;;");
+  g_string_append(scheme, "\n");
+
+#define ADD_TEXT(field)\
+if(directive->field)\
+  {gchar *quote = quote_scheme(directive->field->str);\
+   g_string_append_printf(scheme, "(d-DirectivePut-%s-%s \"%s\" \"%s\")\n",\
+       what, #field, directive->tag->str, quote);\
+   g_free(quote);}
+ADD_TEXT(prefix);			 
+ADD_TEXT(postfix);			 
+ADD_TEXT(display);			 
+ADD_TEXT(graphic_name);
+ADD_TEXT(midibytes);		 
+#undef ADD_TEXT
+#define ADD_INTTEXT(field)\
+if(directive->field)\
+  g_string_append_printf(scheme, "(d-DirectivePut-%s-%s \"%s\" %d)\n",\
+       what, #field, directive->tag->str, directive->field);
+ADD_INTTEXT(minpixels);
+ADD_INTTEXT(override);
+ADD_INTTEXT(x);
+ADD_INTTEXT(y);
+ADD_INTTEXT(tx);
+ADD_INTTEXT(ty);
+ADD_INTTEXT(gx);
+ADD_INTTEXT(gy);
+
+#undef ADD_INTTEXT
+ g_string_append(scheme,  "(d-RefreshDisplay)\n;;;End of scheme script");
+ // quote_scheme(scheme);
+ g_print("Scheme is %s\n", scheme->str);
+  appendSchemeText(scheme->str);
+  g_string_free(scheme, TRUE);
+}
+
+
+
+
+
 /* text_edit_directive
    textually edit the directive via a dialog.
    return FALSE if the user requests deletion of the directive.
 */
-static gboolean text_edit_directive(DenemoDirective *directive) {
+static gboolean text_edit_directive(DenemoDirective *directive, gchar *what) {
   gboolean ret = TRUE;
+#define CREATE_SCRIPT (2)
   DenemoDirective *clone = clone_directive(directive);//for reset
   GtkWidget *dialog = gtk_dialog_new_with_buttons ("Primitive Denemo Directive Edit",
                                         GTK_WINDOW (Denemo.window),
@@ -1353,6 +1529,10 @@ static gboolean text_edit_directive(DenemoDirective *directive) {
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         NULL);
   gtk_dialog_add_button (GTK_DIALOG(dialog), "Delete Directive", GTK_RESPONSE_REJECT);
+  gtk_dialog_add_button (GTK_DIALOG(dialog), "Create Script", CREATE_SCRIPT);
+
+
+
   GtkWidget *vbox = gtk_vbox_new(FALSE, 8);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox,
 		      TRUE, TRUE, 0);
@@ -1412,7 +1592,7 @@ static gboolean text_edit_directive(DenemoDirective *directive) {
   TEXTENTRY("Graphic", graphic_name);
   ADDINTENTRY("Graphic Position", gx, gy);
   TEXTENTRY("Tag", tag);
-
+  TEXTENTRY("MidiBytes", midibytes);
   NEWINTENTRY("Override Mask", override);
 #undef TEXTENTRY
   gtk_widget_show_all (dialog);
@@ -1420,29 +1600,39 @@ static gboolean text_edit_directive(DenemoDirective *directive) {
   g_print("Got response %d\n", response);
 
 
-  if (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT || response ==  GTK_RESPONSE_REJECT){ 
+  if (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT || response ==  GTK_RESPONSE_REJECT){
+    directive->graphic = NULL;//prevent any button being destroyed
     free_directive_data(directive);
     memcpy(directive, clone, sizeof(DenemoDirective));
     if (response ==  GTK_RESPONSE_REJECT) {
       ret = FALSE;//that is it may be deleted, we ensure it has not been changed first,as the tag is used for delelet
     }
   }
-  else
+  else {
+    clone->graphic = NULL;//prevent any button being destroyed
     free_directive(clone);
+  }
 #define REMOVEEMPTIES(field)\
 if(directive->field && directive->field->len==0) g_string_free(directive->field, TRUE), directive->field=NULL;
-  REMOVEEMPTIES( postfix);
+  REMOVEEMPTIES(postfix);
   REMOVEEMPTIES(prefix);
   REMOVEEMPTIES(display);
   REMOVEEMPTIES(graphic_name);
+  
   //REMOVEEMPTIES(tag); don't allow NULL tag
 #undef REMOVEEMPTIES
-
+  
   if(directive->tag && directive->tag->len==0)
     directive->tag = g_string_new(UNKNOWN_TAG);
-  if(directive->graphic_name)
-    loadGraphicItem (directive->graphic_name->str, &directive->graphic,  &directive->width, &directive->height);
+  if(directive->graphic){
+    if(GTK_IS_WIDGET(directive->graphic))
+      label_widget(directive, NULL, NULL);
+    else
+      loadGraphicItem (directive->graphic_name->str, &directive->graphic,  &directive->width, &directive->height);
+  }
   gtk_widget_destroy (dialog);
+  if(response==CREATE_SCRIPT)
+    create_script(directive, what);//g_print("(d-DirectivePut-%s \"%s\")\n", what, directive->tag->str);
   return ret;
 }
 
@@ -1450,18 +1640,22 @@ if(directive->field && directive->field->len==0) g_string_free(directive->field,
    return FALSE if user confirms a request to delete the directive
 */
 static gboolean
-edit_directive(DenemoDirective *directive) {
+edit_directive(DenemoDirective *directive, gchar *what) {
   gboolean ret = TRUE;
   gchar *name = g_strconcat(directive->tag->str, ".scm", NULL);
   gchar* filename = g_build_filename (locatedotdenemo(), "actions", "editscripts", name, NULL);
   if(!g_file_test(filename, G_FILE_TEST_EXISTS)) {
     g_free(filename);
     filename = g_build_filename(get_data_dir(), "actions", "editscripts", name, NULL);
-    if(!g_file_test(filename, G_FILE_TEST_EXISTS)){
-      ret =( text_edit_directive(directive)  || !confirm("Directive Delete", "Are you sure you want to delete the directive?"));
+    if(!g_file_test(filename, G_FILE_TEST_EXISTS)) {
       g_free(filename);
-      score_status (Denemo.gui, TRUE);
-      return ret;
+      filename = g_build_filename (locatedotdenemo(), "download", "actions", "editscripts", name, NULL);
+      if(!g_file_test(filename, G_FILE_TEST_EXISTS)){
+	ret =( text_edit_directive(directive, what)  || !confirm("Directive Delete", "Are you sure you want to delete the directive?"));
+	g_free(filename);
+	score_status (Denemo.gui, TRUE);
+	return ret;
+      }
     }
   }
   GError *error = NULL;
@@ -1474,14 +1668,54 @@ edit_directive(DenemoDirective *directive) {
   return ret;
 }
 
+
+static void edit_directive_callback(GtkWidget *w, gpointer what) {
+  DenemoDirective *directive = g_object_get_data(w, "directive");
+  if(!edit_directive(directive, what)){
+    if(what == score_directive_put_graphic)
+      delete_score_directive(directive->tag->str);
+    else
+      if(what == staff_directive_put_graphic)
+	delete_staff_directive(directive->tag->str);
+      else
+	if(what == voice_directive_put_graphic)
+	  delete_voice_directive(directive->tag->str);
+	else
+	  if(what == scoreheader_directive_put_graphic)
+	    delete_scoreheader_directive(directive->tag->str);
+	  else
+	  if(what == header_directive_put_graphic)
+	    delete_header_directive(directive->tag->str);
+	  else
+	  if(what == paper_directive_put_graphic)
+	    delete_paper_directive(directive->tag->str);
+	  else
+	  if(what == layout_directive_put_graphic)
+	    delete_layout_directive(directive->tag->str);
+	  else
+	  if(what == movementcontrol_directive_put_graphic)
+	    delete_movementcontrol_directive(directive->tag->str);
+	  else
+
+
+
+
+	    g_warning("Not implemented");
+    
+    
+  }    
+ }
+
+
 /**
- * callback for EditDirective 
+ * callback for EditDirective on directive attached to an object.
  */
 void edit_object_directive(GtkAction *action,  DenemoScriptParam *param) {
   //g_print("Edit directive called\n");
   DenemoDirective *directive;
   GList **directives;
-  user_select_directive_at_cursor(&directives, &directive);
+  gchar *what=NULL;
+  user_select_directive_at_cursor(&what, &directives, &directive);
   //g_print("Got directive %p in list %p\n", directive, directives);
   if(directive==NULL) {
     warningdialog("Use the ObjectMenu to modify this object - there are no directives here");
@@ -1489,7 +1723,7 @@ void edit_object_directive(GtkAction *action,  DenemoScriptParam *param) {
   }
   if(directive->tag == NULL)
     directive->tag = g_string_new(UNKNOWN_TAG);
-  if(!edit_directive(directive)) {
+  if(!edit_directive(directive, what)) {
     if(directives && *directives) {
       *directives = g_list_remove(*directives, directive);
       free_directive(directive);
@@ -1506,7 +1740,8 @@ void delete_object_directive(GtkAction *action,  DenemoScriptParam *param) {
   //g_print("Edit directive called\n");
   DenemoDirective *directive;
   GList **directives;
-  user_select_directive_at_cursor(&directives, &directive);
+  gchar *what=NULL;
+  user_select_directive_at_cursor(&what, &directives, &directive);
   //g_print("Got directive %p in list %p\n", directive, directives);
   if(directives==NULL) {
     warningdialog("No directives here");
@@ -1624,7 +1859,7 @@ void edit_voice_directive(GtkAction *action,  DenemoScriptParam *param) {
     return;
   if(directive->tag == NULL)
     directive->tag = g_string_new(UNKNOWN_TAG);
-  if(!edit_directive(directive))
+  if(!edit_directive(directive, "voice"))
     delete_voice_directive(directive->tag->str);
   score_status (Denemo.gui, TRUE);
 }
@@ -1640,7 +1875,7 @@ void edit_staff_directive(GtkAction *action,  DenemoScriptParam *param) {
     return;
   if(directive->tag == NULL)
     directive->tag = g_string_new(UNKNOWN_TAG);
-  if(!edit_directive(directive))
+  if(!edit_directive(directive, "staff"))
     delete_staff_directive(directive->tag->str);
   score_status (Denemo.gui, TRUE);
 }
@@ -1656,7 +1891,7 @@ void edit_clef_directive(GtkAction *action,  DenemoScriptParam *param) {
     return;
   if(directive->tag == NULL)
     directive->tag = g_string_new(UNKNOWN_TAG);
-  if(!edit_directive(directive))
+  if(!edit_directive(directive, "clef"))
     delete_clef_directive(directive->tag->str);
   score_status (Denemo.gui, TRUE);
 }
@@ -1671,7 +1906,7 @@ void edit_keysig_directive(GtkAction *action,  DenemoScriptParam *param) {
     return;
   if(directive->tag == NULL)
     directive->tag = g_string_new(UNKNOWN_TAG);
-  if(!edit_directive(directive))
+  if(!edit_directive(directive, "keysig"))
     delete_keysig_directive(directive->tag->str);
   score_status (Denemo.gui, TRUE);
 }
@@ -1688,7 +1923,7 @@ void edit_timesig_directive(GtkAction *action,  DenemoScriptParam *param) {
     return;
   if(directive->tag == NULL)
     directive->tag = g_string_new(UNKNOWN_TAG);
-  if(!edit_directive(directive))
+  if(!edit_directive(directive, "timesig"))
     delete_timesig_directive(directive->tag->str);
   score_status (Denemo.gui, TRUE);
 }
@@ -1729,7 +1964,7 @@ void edit_score_directive(GtkAction *action,  DenemoScriptParam *param) {
       return;\
     if(directive->tag == NULL)\
       directive->tag = g_string_new(UNKNOWN_TAG);\
-    if(!edit_directive(directive))\
+    if(!edit_directive(directive, #what))\
       delete_##what##_directive(directive->tag->str);\
   score_status (Denemo.gui, TRUE);\
   }
@@ -1782,7 +2017,7 @@ void edit_movement_directive(GtkAction *action,  DenemoScriptParam *param) {
       return;\
     if(directive->tag == NULL)\
       directive->tag = g_string_new(UNKNOWN_TAG);\
-    if(!edit_directive(directive))\
+    if(!edit_directive(directive, #what))\
       delete_##what##_directive(directive->tag->str);\
   score_status (Denemo.gui, TRUE);\
   }
@@ -1906,11 +2141,11 @@ GET_INT_FIELD_FUNC(scoreheader, override)
 GET_INT_FIELD_FUNC(scoreheader, width)
 GET_INT_FIELD_FUNC(scoreheader, height)
 
-PUT_GRAPHIC(scoreheader)
+     //PUT_GRAPHIC(scoreheader)
 
 PUT_STR_FIELD_FUNC(scoreheader, prefix)
 PUT_STR_FIELD_FUNC(scoreheader, postfix)
-PUT_STR_FIELD_FUNC(scoreheader, display)
+
 
 GET_STR_FIELD_FUNC(scoreheader, prefix)
 GET_STR_FIELD_FUNC(scoreheader, postfix)
@@ -1934,11 +2169,11 @@ GET_INT_FIELD_FUNC(header, override)
 GET_INT_FIELD_FUNC(header, width)
 GET_INT_FIELD_FUNC(header, height)
 
-PUT_GRAPHIC(header)
+     //PUT_GRAPHIC(header)
 
 PUT_STR_FIELD_FUNC(header, prefix)
 PUT_STR_FIELD_FUNC(header, postfix)
-PUT_STR_FIELD_FUNC(header, display)
+
 
 GET_STR_FIELD_FUNC(header, prefix)
 GET_STR_FIELD_FUNC(header, postfix)
@@ -1962,11 +2197,11 @@ GET_INT_FIELD_FUNC(paper, override)
 GET_INT_FIELD_FUNC(paper, width)
 GET_INT_FIELD_FUNC(paper, height)
 
-PUT_GRAPHIC(paper)
+     //PUT_GRAPHIC(paper)
 
 PUT_STR_FIELD_FUNC(paper, prefix)
 PUT_STR_FIELD_FUNC(paper, postfix)
-PUT_STR_FIELD_FUNC(paper, display)
+
 
 GET_STR_FIELD_FUNC(paper, prefix)
 GET_STR_FIELD_FUNC(paper, postfix)
@@ -1988,11 +2223,11 @@ GET_INT_FIELD_FUNC(layout, override)
 GET_INT_FIELD_FUNC(layout, width)
 GET_INT_FIELD_FUNC(layout, height)
 
-PUT_GRAPHIC(layout)
+     //PUT_GRAPHIC(layout)
 
 PUT_STR_FIELD_FUNC(layout, prefix)
 PUT_STR_FIELD_FUNC(layout, postfix)
-PUT_STR_FIELD_FUNC(layout, display)
+
 
 GET_STR_FIELD_FUNC(layout, prefix)
 GET_STR_FIELD_FUNC(layout, postfix)
@@ -2017,11 +2252,11 @@ GET_INT_FIELD_FUNC(movementcontrol, override)
 GET_INT_FIELD_FUNC(movementcontrol, width)
 GET_INT_FIELD_FUNC(movementcontrol, height)
 
-PUT_GRAPHIC(movementcontrol)
+     //PUT_GRAPHIC(movementcontrol)
 
 PUT_STR_FIELD_FUNC(movementcontrol, prefix)
 PUT_STR_FIELD_FUNC(movementcontrol, postfix)
-PUT_STR_FIELD_FUNC(movementcontrol, display)
+
 
 GET_STR_FIELD_FUNC(movementcontrol, prefix)
 GET_STR_FIELD_FUNC(movementcontrol, postfix)
@@ -2051,3 +2286,26 @@ gchar * get_scoretitle(void){
   }
   return scoretitle;
 }
+
+
+#define TEXT_EDIT_DIRECTIVE(what)\
+gboolean text_edit_##what##_directive(gchar *tag) {\
+  DenemoDirective *directive = get_##what##_directive(tag);\
+  if(directive)\
+    return text_edit_directive(directive, #what);\
+  return FALSE;\
+}
+TEXT_EDIT_DIRECTIVE(note);
+TEXT_EDIT_DIRECTIVE(chord);
+TEXT_EDIT_DIRECTIVE(staff);
+TEXT_EDIT_DIRECTIVE(voice);
+TEXT_EDIT_DIRECTIVE(score);
+TEXT_EDIT_DIRECTIVE(clef);
+TEXT_EDIT_DIRECTIVE(timesig);
+TEXT_EDIT_DIRECTIVE(keysig);
+TEXT_EDIT_DIRECTIVE(scoreheader);
+TEXT_EDIT_DIRECTIVE(header);
+TEXT_EDIT_DIRECTIVE(paper);
+TEXT_EDIT_DIRECTIVE(layout);
+TEXT_EDIT_DIRECTIVE(movementcontrol);
+#undef TEXT_EDIT_DIRECTIVE
