@@ -414,8 +414,6 @@ main (int argc, char *argv[])
   /* gtk initialization */
   gtk_init (&argc, &argv);
 
-  
-
 //#ifdef G_OS_WIN32
 //  /* workaround necessary for compilation on Cygwin */
 //  g_set_print_handler ((GPrintFunc)printf);
@@ -625,7 +623,7 @@ COPYING for details.\n\n") ;
           GtkWidget *label =
             gtk_label_new
             ("\nDenemo crashed, The open file has been recovered\n"
-             "do you want to contiue editing your work?\n");
+             "do you want to continue editing your work?\n");
           gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
                              label);
           gtk_widget_show_all (dialog);
@@ -684,5 +682,5 @@ COPYING for details.\n\n") ;
           return 1;
         } 
     } else
-      open_user_default_template();
+      open_user_default_template(REPLACE_SCORE);
 }
