@@ -22,32 +22,31 @@ initkeyaccs (gint * accs, gint number);
 void
 freeobject (DenemoObject * mudobj)
 {
-  if (mudobj)
-    switch (mudobj->type)
-      {
-      case CHORD:
-	freechord (mudobj);	/* Which also frees mudobj itself */
-	break;
-      case CLEF:
-	free_directives(((clef*)mudobj->object)->directives);
-	g_free (mudobj);
-	break;
-
-      case KEYSIG:
-	free_directives(((keysig*)mudobj->object)->directives);
-	g_free (mudobj);
-	break;
-      case TIMESIG:
-	free_directives(((timesig*)mudobj->object)->directives);
-	g_free (mudobj);
-	break;
-
-
-
-      default:
-	g_free (mudobj);
-	break;
-      }
+  if (mudobj==NULL)
+    return;
+  if(mudobj->midi_events)
+    g_list_free(mudobj->midi_events);//do not free the data it belongs to libsmf
+  switch (mudobj->type)
+    {
+    case CHORD:
+      freechord (mudobj);	/* Which also frees mudobj itself */
+      break;
+    case CLEF:
+      free_directives(((clef*)mudobj->object)->directives);
+      g_free (mudobj);
+      break;
+    case KEYSIG:
+      free_directives(((keysig*)mudobj->object)->directives);
+      g_free (mudobj);
+      break;
+    case TIMESIG:
+      free_directives(((timesig*)mudobj->object)->directives);
+      g_free (mudobj);
+      break; 
+    default:
+      g_free (mudobj);
+      break;
+    } 
 }
 
 /**
@@ -215,7 +214,9 @@ DenemoDirective *clone_directive(DenemoDirective *directive) {
   memcpy(ret, directive, sizeof(DenemoDirective));//BEWARE all pointers in DenemoDirective require code, as follows:
 #define CLONE(field) \
       if(directive->field && directive->field->len)\
-        ret->field = g_string_new(directive->field->str);
+        ret->field = g_string_new(directive->field->str);\
+      else\
+        ret->field = NULL;
   CLONE(tag);
   CLONE(prefix);
   CLONE(postfix);
@@ -244,8 +245,12 @@ free_directive_data(DenemoDirective *directive) {
     DFREE(postfix);
     DFREE(graphic_name);
 #undef DFREE
-    //if(directive->graphic)
-    //  g_object_unref(directive->graphic); we leave these in a hash table now, and never discard them
+    if(directive->graphic && GTK_IS_WIDGET(directive->graphic)) {
+      //g_print("We should destroy the widget now");
+      gtk_widget_destroy(directive->graphic);
+    }
+      else	
+   ; //  g_object_unref(directive->graphic); we leave these GdkBitmap in a hash table, and never discard them
 }
 void
 free_directive(DenemoDirective *directive) {
@@ -322,6 +327,7 @@ dnm_clone_object (DenemoObject * orig)
 	      break;
 	    }
   }
+  ret->midi_events = NULL;
   return ret;
 }
 
