@@ -695,7 +695,7 @@ gint pitchentry(DenemoGUI *gui) {
       if(!PR_tuning){
 	display_pitch(note, gui);
 	if(gui->input_source==INPUTMIDI)
-	  play_pitch(found->pitch * (pow(2,(octave))), 0.3, 0.5, 0);
+	  playpitch(found->pitch * (pow(2,(octave))), 0.3, 0.5, 0);
 	if(!Denemo.prefs.overlays) {
 	  enter_note_in_score(gui, found, octave);
 	  if(gui->mode & INPUTRHYTHM) {
