@@ -32,8 +32,11 @@ struct callbackdata
   GtkWidget *autosave;
   GtkWidget *notation_palette;
   GtkWidget *rhythm_palette;
+  GtkWidget *object_palette;
   GtkWidget *articulation_palette;
-  GtkWidget *visible_titles;
+  GtkWidget *visible_directive_buttons;
+  GtkWidget *autoupdate;
+
   GtkWidget *autosave_timeout;
   GtkWidget *maxhistory;
   GtkWidget *browser;
@@ -159,9 +162,11 @@ set_preferences (struct callbackdata *cbdata)
   ASSIGNBOOLEAN(autosave)
   ASSIGNINT(autosave_timeout)
   ASSIGNBOOLEAN(articulation_palette)
-  ASSIGNBOOLEAN(visible_titles)
+  ASSIGNBOOLEAN(visible_directive_buttons)
+  ASSIGNBOOLEAN(autoupdate)
   ASSIGNBOOLEAN(notation_palette)
   ASSIGNBOOLEAN(rhythm_palette)
+  ASSIGNBOOLEAN(object_palette)
   ASSIGNBOOLEAN(saveparts)
   //g_print ("Timeout %d \n", prefs->autosave_timeout);
 
@@ -261,8 +266,11 @@ preferences_change (GtkAction *action, gpointer param)
   BOOLEANENTRY("Play back entered notes immediately", immediateplayback);  
   BOOLEANENTRY("Display duration toolbar", notation_palette);
   BOOLEANENTRY("Display articulation palette", articulation_palette);
-  BOOLEANENTRY("Display Score Titles", visible_titles);
+  BOOLEANENTRY("Display Titles. Controls etc", visible_directive_buttons);
+
   BOOLEANENTRY("Display rhythm pattern toolbar", rhythm_palette);
+  BOOLEANENTRY("Display menu of objects toolbar", object_palette);
+
 
   hbox = gtk_hbox_new (FALSE, 8);
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);
@@ -315,7 +323,7 @@ preferences_change (GtkAction *action, gpointer param)
   TEXTENTRY("Image Viewer", imageviewer)
   TEXTENTRY("Text Editor", texteditor)
   TEXTENTRY("Default Save Path", denemopath)
-
+  BOOLEANENTRY("Update the command set on startup", autoupdate);
   /*
    * Plugins settings
    */
