@@ -259,7 +259,7 @@ process_lilypond_errors(gchar *filename){
 	warningdialog("Spurious line number"), line = 0;
       /* gchar *errmsg = g_strdup_printf("Error at line %d column %d %d", line,column, cnv); */
       /*     warningdialog(errmsg); */
-      infodialog(epoint);
+      infodialog(epoint);/* FIXME PUT OUTPUT IN CONSOLE WINDOW HERE */
       if(gui->textbuffer) {
 	set_lily_error(line+1, column, gui);
       } 
@@ -878,6 +878,8 @@ void refresh_print_view (void) {
   gchar *filename = get_printfile_pathbasename();
   gchar *lilyfile = g_strconcat (filename, "_.ly", NULL);
   remove (lilyfile);
+  gchar *path = g_strconcat (filename, "_.png", NULL);
+  remove (path);
   gui->si->markstaffnum=0;//remove selection, as exportlilypond respects it - FIXME??
   exportlilypond (lilyfile, gui,  TRUE);
   convert_ly(lilyfile);
@@ -996,11 +998,16 @@ printarea_configure_event (GtkWidget * widget, GdkEventConfigure * event)
   gtk_adjustment_changed(hadjust);
 }
 
+static gint adjust_x=0;
+static gint adjust_y=0;
+
+
 static void
 printvertical_scroll (GtkAdjustment * adjust)
 {
   DenemoGUI *gui = Denemo.gui;
   // g_print("vertical %d to %d\n", (int)adjust->value, (int)(adjust->value+adjust->page_size));
+  adjust_y=(int)adjust->value;
   gtk_widget_queue_draw (gui->printarea);
 }
 
@@ -1009,6 +1016,7 @@ printhorizontal_scroll (GtkAdjustment * adjust)
 {
   DenemoGUI *gui = Denemo.gui;
   // g_print("horizontal %d to %d\n", (int)adjust->value, (int)(adjust->value+adjust->page_size));
+  adjust_x=(int)adjust->value;
 gtk_widget_queue_draw (gui->printarea);
 }
 
@@ -1146,10 +1154,11 @@ printarea_button_release (GtkWidget * widget, GdkEventButton * event)
 
     GdkPixbuf *alphapixbuf = gdk_pixbuf_add_alpha (Denemo.gui->pixbuf, TRUE, 255, 255, 255);
     if(alphapixbuf){
-      gchar *data =  create_xbm_data_from_pixbuf(alphapixbuf, markx, marky, pointx, pointy);
-  
+      //   gchar *data =  create_xbm_data_from_pixbuf(alphapixbuf, markx, marky, pointx, pointy);
+      gchar *data =  create_xbm_data_from_pixbuf(alphapixbuf, markx+adjust_x, marky+adjust_y, pointx+adjust_x, pointy+adjust_y);
+      g_print("Used adjust x y %d, %d\n", adjust_x, adjust_y);
       GtkIconFactory *icon_factory = gtk_icon_factory_new ();
-      GdkPixbuf *sub_pixbuf = gdk_pixbuf_new_subpixbuf (Denemo.gui->pixbuf, markx, marky, width, height);
+      GdkPixbuf *sub_pixbuf = gdk_pixbuf_new_subpixbuf (Denemo.gui->pixbuf, markx+adjust_x, marky+adjust_y, width, height);
 
       GtkIconSet *icon_set = gtk_icon_set_new_from_pixbuf (sub_pixbuf);
       g_object_unref(sub_pixbuf);
