@@ -879,6 +879,10 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "InsertMovementBefore")),
 SCM scheme_InsertMovementAfter(SCM optional);
 install_scm_function ("d-InsertMovementAfter", scheme_InsertMovementAfter);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "InsertMovementAfter")), "scm", (gpointer)1);
+/*NewMovement append_new_movement*/
+SCM scheme_NewMovement(SCM optional);
+install_scm_function ("d-NewMovement", scheme_NewMovement);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "NewMovement")), "scm", (gpointer)1);
 /*SaveParts file_savepartswrapper*/
 SCM scheme_SaveParts(SCM optional);
 install_scm_function ("d-SaveParts", scheme_SaveParts);
@@ -935,6 +939,10 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "Cut")), "scm", (gpointer)
 SCM scheme_Paste(SCM optional);
 install_scm_function ("d-Paste", scheme_Paste);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "Paste")), "scm", (gpointer)1);
+/*PasteClipboard paste_clipboard*/
+SCM scheme_PasteClipboard(SCM optional);
+install_scm_function ("d-PasteClipboard", scheme_PasteClipboard);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "PasteClipboard")), "scm", (gpointer)1);
 /*ScoreProperties score_properties_dialog*/
 SCM scheme_ScoreProperties(SCM optional);
 install_scm_function ("d-ScoreProperties", scheme_ScoreProperties);
@@ -1063,10 +1071,14 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "ChangeNotehead")), "scm",
 SCM scheme_InsertStem(SCM optional);
 install_scm_function ("d-InsertStem", scheme_InsertStem);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "InsertStem")), "scm", (gpointer)1);
-/*EditLyric lyric_insert*/
-SCM scheme_EditLyric(SCM optional);
-install_scm_function ("d-EditLyric", scheme_EditLyric);
-g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "EditLyric")), "scm", (gpointer)1);
+/*AddVerse add_verse*/
+SCM scheme_AddVerse(SCM optional);
+install_scm_function ("d-AddVerse", scheme_AddVerse);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "AddVerse")), "scm", (gpointer)1);
+/*DeleteVerse delete_verse*/
+SCM scheme_DeleteVerse(SCM optional);
+install_scm_function ("d-DeleteVerse", scheme_DeleteVerse);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "DeleteVerse")), "scm", (gpointer)1);
 /*EditFiguredBass figure_insert*/
 SCM scheme_EditFiguredBass(SCM optional);
 install_scm_function ("d-EditFiguredBass", scheme_EditFiguredBass);
@@ -1195,6 +1207,10 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoreCommands")), "scm", (
 SCM scheme_MyCommands(SCM optional);
 install_scm_function ("d-MyCommands", scheme_MyCommands);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MyCommands")), "scm", (gpointer)1);
+/*FetchCommands fetchcommands*/
+SCM scheme_FetchCommands(SCM optional);
+install_scm_function ("d-FetchCommands", scheme_FetchCommands);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "FetchCommands")), "scm", (gpointer)1);
 /*AddBookmark addbookmark*/
 SCM scheme_AddBookmark(SCM optional);
 install_scm_function ("d-AddBookmark", scheme_AddBookmark);
