@@ -28,7 +28,10 @@ point_to_empty_movement (DenemoGUI *gui)
 {
   DenemoScore *newscore = (DenemoScore *) g_malloc0 (sizeof (DenemoScore));
   init_score (newscore, gui);
+  if(gui->si && gui->si->buttonbox)
+     gtk_widget_hide(gui->si->buttonbox);
   gui->si = newscore;
+  gtk_widget_show(gui->si->buttonbox);
 }
 /**
  * Create new DenemoScore with one empty staff
@@ -49,6 +52,32 @@ new_movement(DenemoGUI *gui, gboolean before) {
     gui->movements = g_list_prepend(gui->movements, gui->si);//FIXME insert before g_list_find(gui->movements, si)
   else
     gui->movements = g_list_append(gui->movements, gui->si);//ditto after
+  // It would be good to keep a record of the template used to open the music (relative to templates) and try and open that
+  if(open_user_default_template(ADD_MOVEMENTS))   {//no default template
+    set_width_to_work_with(gui);
+    //FIXME duplicate code
+    set_rightmeasurenum (gui->si);
+    find_leftmost_allcontexts (gui->si);
+    set_bottom_staff (gui);
+    update_hscrollbar (gui);
+    update_vscrollbar (gui);
+    gtk_widget_queue_draw (gui->scorearea);
+    gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
+    gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
+    displayhelper(gui);
+    score_status(gui, TRUE);
+  }
+}
+
+
+void
+append_new_movement(GtkAction *action, gpointer param) {
+  DenemoGUI *gui = Denemo.gui;
+  //It would be good to keep a record of the template used to open the music (relative to templates) and try and open that
+  if(open_user_default_template(ADD_MOVEMENTS))   {//no default template
+    point_to_new_movement(gui);
+    gui->movements = g_list_append(gui->movements, gui->si);
+  }
   set_width_to_work_with(gui);
   //FIXME duplicate code
   set_rightmeasurenum (gui->si);
@@ -60,9 +89,10 @@ new_movement(DenemoGUI *gui, gboolean before) {
   gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
   gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
   displayhelper(gui);
-  score_status(gui, TRUE);
+  score_status(gui, TRUE); 
 }
 
+
 void
 insert_movement_before(GtkAction *action, gpointer param) {
   DenemoGUI *gui = Denemo.gui;
@@ -119,7 +149,9 @@ goto_movement_staff_obj (DenemoGUI * gui, gint movementnum, gint staffnum, gint
     warningdialog(_("No such movement"));
     return FALSE;
   }
+  gtk_widget_hide(gui->si->buttonbox);
   gui->si = this->data;
+  gtk_widget_show(gui->si->buttonbox);
   if(!set_currentstaffnum (gui, staffnum))
     {
       warningdialog(_("No such voice"));
@@ -173,8 +205,9 @@ next_movement (GtkAction *action, DenemoScriptParam *param)
       warningdialog(_("This is the last movement"));
     return;
   }
+  gtk_widget_hide(gui->si->buttonbox);
   gui->si = this->data;
-
+  gtk_widget_show(gui->si->buttonbox);
   updatescoreinfo (gui);
   //FIXME duplicate code
   set_rightmeasurenum (gui->si);
@@ -209,7 +242,9 @@ prev_movement (GtkAction *action, DenemoScriptParam *param)
       warningdialog(_("This is the first movement"));
     return;
   }
+  gtk_widget_hide(gui->si->buttonbox);
   gui->si = this->data; 
+  gtk_widget_show(gui->si->buttonbox);
 
   updatescoreinfo (gui);
   //FIXME duplicate code
@@ -280,17 +315,12 @@ init_score (DenemoScore * si, DenemoGUI *gui)
   si->undodata = g_queue_new ();
   si->redodata = g_queue_new ();
   si->undo_redo_mode = UNDO;
-  if(gui->movements) {
-    //{
-    //static int count;
-    //    if(count++==0)
-    //      warningdialog("It should really copy your staff structures over - sorry!");
-    //    }
-    // FIXME initialize from current movement    g_list_find(gui->movements, si)
-    //gint number = g_list_index( gui->movements, gui->si);
-    //g_string_printf(si->headerinfo.piece, "Movement %d", 1+g_list_length(gui->movements));
-  }
-  //gui->movements = g_list_append(gui->movements, si);
+  
+  si->buttonbox = gtk_hbox_new(FALSE, 1);
+  gtk_box_pack_end (GTK_BOX (gui->buttonboxes), si->buttonbox, FALSE, TRUE,
+		      0);
+  gtk_widget_show (si->buttonbox);
+  GTK_WIDGET_UNSET_FLAGS(si->buttonbox, GTK_CAN_FOCUS);
 }
 
 static delete_all_staffs(DenemoGUI * gui) {
@@ -318,8 +348,15 @@ free_score (DenemoGUI * gui)
   delete_all_staffs(gui);
   delete_directives(&gui->si->layout.directives);
   delete_directives(&gui->si->header.directives);
-
-
+  if(gui->si->buttonbox) {
+    gtk_widget_destroy(gui->si->buttonbox);
+    gui->si->buttonbox = NULL;
+  }
+  if(gui->si->lyricsbox) {
+    gtk_widget_destroy(gui->si->lyricsbox);
+    gui->si->lyricsbox = NULL;
+  }
+  reset_lyrics(NULL, 0);
   g_queue_free (gui->si->undodata);
   g_queue_free (gui->si->redodata);
 }
