@@ -58,6 +58,8 @@ void redowrapper(GtkAction *action, gpointer param);
 void update_undo_info(DenemoScore *si, unre_data *undo);
 void update_redo_info(DenemoScore *si, unre_data *redo);
 void goto_mark (GtkAction *action, gpointer param);
+DenemoObject *get_mark_object(void);
+DenemoObject *get_point_object(void);
 void save_selection(DenemoScore *si);
 void restore_selection(DenemoScore *si);
 gboolean in_selection(DenemoScore *si);
