@@ -17,9 +17,10 @@
 #include "staffops.h"
 #include <stdlib.h>
 #include <string.h>
-#include <gtk/gtk.h>
 #include "calculatepositions.h"
 #include "commandfuncs.h"
+#include "lilydirectives.h"
+
 #ifdef _HAVE_JACK_
 #include "jackmidi.h"
 #endif
@@ -186,6 +187,23 @@ newstaff (DenemoGUI * gui, enum newstaffcallbackaction action,
   gint ret = -1, err;
   DenemoStaff *thestaffstruct =
     (DenemoStaff *) g_malloc (sizeof (DenemoStaff));
+
+  //CREATE a GtkMenus to popup when clicking to left of the staff
+  {
+    //hmm, cannot populate it, why not ? FIXME 
+    GtkWidget *item;
+    thestaffstruct->staffmenu = gtk_menu_new();
+    item = gtk_menu_item_new_with_label("Edit Attributes");
+    gtk_menu_shell_append(GTK_MENU_SHELL( thestaffstruct->staffmenu), item);
+    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(edit_staff_directive), NULL);
+    gtk_widget_show(item);
+    thestaffstruct->voicemenu = gtk_menu_new();
+    item = gtk_menu_item_new_with_label("Edit Attributes");
+    gtk_menu_shell_append(GTK_MENU_SHELL( thestaffstruct->voicemenu), item);
+    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(edit_voice_directive), NULL);
+    gtk_widget_show(item);
+  }
+
   struct newstaffinfotopass itp;
   measurenode *themeasures = NULL;	/* Initial set of measures in staff */
   gint numstaffs = g_list_length (si->thescore);
@@ -421,11 +439,14 @@ deletestaff (DenemoGUI * gui, gboolean interactive)
     
   gboolean isprimary = ((int) curstaffstruct->voicenumber == 1);
   //FIXME free_staff()
+  gtk_widget_destroy((curstaffstruct->staffmenu));
+  gtk_widget_destroy((curstaffstruct->voicemenu));
   g_list_foreach (curstaffstruct->measures, freeobjlist, NULL);
   g_list_free (curstaffstruct->measures);
   g_string_free (curstaffstruct->denemo_name, FALSE);//FIXME these should all be TRUE??
   g_string_free (curstaffstruct->lily_name, FALSE);
   g_string_free (curstaffstruct->midi_instrument, FALSE);
+  g_list_foreach (curstaffstruct->verses, gtk_widget_destroy, NULL);
   g_free (curstaffstruct);
 
   free_directives(curstaffstruct->staff_directives);
