@@ -12,6 +12,7 @@
 #include <denemo/denemo.h>
 #include "notewidths.h"
 #include "utils.h"
+#include "smf.h"
 #include <signal.h> /*for SIGTERM */
 
 #include "config.h"
@@ -603,7 +604,7 @@ initdir ()
 {
 #ifndef G_OS_WIN32
   GError *error=NULL;
-  if(!gbr_init (&error))
+  if(!gbr_init (&error)&& (error != GBR_INIT_ERROR_DISABLED))
     {
       g_print ("BinReloc failed to initialize:\n");
       g_print ("Domain: %d (%s)\n",
@@ -724,7 +725,7 @@ void  set_title_bar(DenemoGUI *gui) {
     title = gui->filename->str;
   else
     title = "(Untitled)";
-  title = g_strdup_printf("Denemo - %s%c", title, gui->changecount?'*':' ');
+  title = g_strdup_printf("Denemo - %s%c", title, gui->notsaved?'*':' ');
   gtk_window_set_title (GTK_WINDOW (Denemo.window), title); 
   gchar *base = g_path_get_basename (title);
   gtk_notebook_set_tab_label_text (GTK_NOTEBOOK(Denemo.notebook), gui->page, base); 
@@ -739,13 +740,15 @@ void  set_title_bar(DenemoGUI *gui) {
                    FALSE = the score is to be assumed saved
 */
 void score_status(DenemoGUI *gui, gboolean change) {
-  if(change) { 
+  if(change) {
+    gboolean just_changed = !gui->notsaved;
+    gui->notsaved = TRUE; 
     gui->changecount++;
-    if(gui->changecount==1)//just changed
+    gui->si->changecount++;
+    if(just_changed)
       set_title_bar(gui);
-
   } else {
-    gui->changecount=0;
+    gui->notsaved=FALSE;
     set_title_bar(gui);
   }
   write_status(gui);
@@ -757,11 +760,21 @@ void score_status(DenemoGUI *gui, gboolean change) {
 ********************/
 
 void write_status(DenemoGUI *gui) {
+  gint minutes = 0;
+  gdouble seconds = 0.0;
+
   gchar *selection;
   if(gui->si==NULL)
     return;
   if(gui->si->currentobject && gui->si->currentobject->data ) {
     DenemoObject *curObj = gui->si->currentobject->data;
+    if((gui->si->smfsync == gui->si->changecount) && curObj->midi_events) {
+      smf_event_t *event = (smf_event_t*)curObj->midi_events->data;
+      gdouble time = event->time_seconds;
+      minutes = time/60.0;
+      seconds = time - 60*minutes;
+    }
+
     switch(curObj->type) {
     case CHORD: {
       chord *thechord =  ((chord *) curObj->object);
@@ -866,7 +879,7 @@ void write_status(DenemoGUI *gui) {
 
   status = "Movement";
   gint index = g_list_index(gui->movements, gui->si);
-  status = g_strdup_printf("%s %d: %s", status, index+1, selection);
+  status = g_strdup_printf("%s %d: %s: %d min %.2f sec", status, index+1, selection, minutes, seconds);
 
   g_free(selection);
   gtk_statusbar_pop(GTK_STATUSBAR (Denemo.statusbar), Denemo.status_context_id);
