@@ -45,8 +45,7 @@
 #define EXTRA_ACCELS "extra.accels"
 
 
-/* void newview(GtkAction*, gpointer param); */
-
+void newtab (GtkAction *action, gpointer param);
 void free_gui(DenemoGUI *gui);/* frees all movement data from gui, leaving gui interface intact */
 void addhistorymenuitem(gchar *filename);
 
